@@ -1,10 +1,10 @@
-;;; cg.el --- major mode for editing Constraint Grammar files  -*- lexical-binding: t; coding: utf-8 -*-
+;;; cg.el --- Major mode for editing Constraint Grammar files  -*- lexical-binding: t; coding: utf-8 -*-
 
-;; Copyright (C) 2010-2018 Kevin Brubeck Unhammer
+;; Copyright (C) 2010-2022 Kevin Brubeck Unhammer
 
 ;; Author: Kevin Brubeck Unhammer <unhammer@fsfe.org>
-;; Version: 0.3.1
-;; Package-Requires: ((emacs "24.3"))
+;; Version: 0.4.0
+;; Package-Requires: ((emacs "26.1"))
 ;; Url: https://visl.sdu.dk/constraint_grammar.html
 ;; Keywords: languages
 
@@ -42,8 +42,8 @@
 
 ;; I recommend using `company-mode' for tab-completion, and
 ;; `smartparens-mode' if you're used to it (`paredit-mode' does not
-;; work well if you have set names with the # character in them). Both
-;; are available from MELPA (see http://melpa.milkbox.net/).
+;; work well if you have set names with the # character in them).
+;; Both are available from MELPA (see http://melpa.milkbox.net/).
 ;;
 ;; You can lazy-load company-mode for cg-mode like this:
 ;;
@@ -55,22 +55,21 @@
 ;; - investigate bug in `show-smartparens-mode' causing slowness
 ;; - different syntax highlighting for sets and tags (difficult)
 ;; - use something like prolog-clause-start to define M-a/e etc.
-;; - run vislcg3 --show-unused-sets and buttonise with line numbers (like Occur does)
 ;; - indentation function (based on prolog again?)
 ;; - the rest of the keywords
 ;; - https://visl.sdu.dk/cg3/single/#regex-icase
 ;; - keyword tab-completion
 ;; - `font-lock-syntactic-keywords' is obsolete since 24.1
-;; - goto-set/list
 ;; - show definition of set/list-at-point in modeline
 ;; - show section name/number in modeline
 
 ;;; Code:
 
-(defconst cg-version "0.3.1" "Version of cg-mode.")
+(defconst cg-version "0.4.0" "Version of cg-mode.")
 
-(eval-when-compile (require 'cl))
-(require 'cl-lib)
+(eval-when-compile (require 'cl-lib))
+(require 'xref)
+(require 'tool-bar)
 
 ;;;============================================================================
 ;;;
@@ -274,13 +273,74 @@ Don't change without re-evaluating the file.")
     table)
   "Syntax table for CG mode.")
 
+
+
+;;; Tool-bar -----------------------------------------------------------------
+
+(defvar cg-mode-tool-bar-map
+  (when (keymapp tool-bar-map)
+    (let ((map (copy-keymap tool-bar-map)))
+      (define-key-after map [separator-cg] menu-bar-separator)
+      (tool-bar-local-item
+       "show" 'cg-edit-input 'cg-edit-input map
+       :help "Edit input examples")
+      (tool-bar-local-item
+       "spell" 'cg-check 'cg-check map
+       :help "Check input with grammar")
+      (define-key-after map [separator-cg-2] menu-bar-separator)
+      (tool-bar-local-item
+       "sort-criteria" 'cg-output-set-unhide 'cg-output-set-unhide map
+       :help "Show only matching analyses in output")
+      (tool-bar-local-item
+       "describe" 'cg-output-toggle-analyses 'cg-output-toggle-analyses map
+       :help "Toggle analyses in output")
+      map)))
+
+(defvar cg-input-mode-tool-bar-map
+  (when (keymapp tool-bar-map)
+    (let ((map (copy-keymap tool-bar-map)))
+      (define-key-after map [separator-cg] menu-bar-separator)
+      (tool-bar-local-item
+       "show" 'cg-back-to-file 'cg-back-to-file map
+       :help "Back to grammar rules file")
+      (tool-bar-local-item
+       "spell" 'cg-back-to-file-and-check 'cg-back-to-file-and-check map
+       :help "Check input with grammar")
+      map)))
+
+(defvar cg-output-mode-tool-bar-map
+  (when (keymapp tool-bar-map)
+    (let ((map (copy-keymap tool-bar-map)))
+      (define-key-after map [separator-cg] menu-bar-separator)
+      (tool-bar-local-item
+       "show" 'cg-back-to-file-and-edit-input 'cg-back-to-file-and-edit-input map
+       :help "Back to grammar rules file")
+      (tool-bar-local-item
+       "spell" 'cg-back-to-file-and-check 'cg-back-to-file-and-check map
+       :help "Check input examples with grammar")
+      (define-key-after map [separator-cg-2] menu-bar-separator)
+      (tool-bar-local-item
+       "sort-criteria" 'cg-output-set-unhide 'cg-output-set-unhide map
+       :help "Show only matching analyses in output")
+      (tool-bar-local-item
+       "describe" 'cg-output-toggle-analyses 'cg-output-toggle-analyses map
+       :help "Toggle analyses in output")
+      map)))
+
+
+
+
+;;; Navigation/comments ------------------------------------------------------
+
 (defun cg-beginning-of-defun ()
+  "Go to beginning of a rule."
   (re-search-backward defun-prompt-regexp nil 'noerror)
   (while (nth 4 (syntax-ppss))
     (re-search-backward defun-prompt-regexp nil 'noerror))
   (re-search-backward "\"<[^\"]>\"" (line-beginning-position) 'noerror))
 
 (defun cg-end-of-defun ()
+  "Go to end of a rule."
   (and (search-forward ";")
        (re-search-forward defun-prompt-regexp nil 'noerror)
        (goto-char (match-beginning 0)))
@@ -291,11 +351,13 @@ Don't change without re-evaluating the file.")
   (re-search-backward "\"<[^\"]>\"" (line-beginning-position) 'noerror))
 
 (defun cg--line-commented-p ()
+  "Check if this line is commented or not."
   (save-excursion
     (back-to-indentation)
     (looking-at "#")))
 
 (defun cg--region-commented-p (beg end)
+  "Check if the region BEG .. END is *completely* commented or not."
   (catch 'ret
     (save-excursion
       (goto-char beg)
@@ -306,13 +368,14 @@ Don't change without re-evaluating the file.")
           (throw 'ret nil)))
       (throw 'ret t))))
 
-(defun cg--comment/uncomment-rule (comment &optional n)
-  "Comment/uncomment a rule around point."
+(defun cg--comment-uncomment-rule (comment &optional n)
+  "Toggle whether rule-at-point is commented out or not.
+If COMMENT, always comment out.  If N is a number, do that many rules."
   (let ((i 0)
         (n (if (numberp n) n 1))
         (initial-point (point-marker)))
     (while (< i n)
-      (incf i)
+      (cl-incf i)
       (let* ((r (save-excursion
                   (if (search-forward ";" nil 'noerror)
                       (1+ (point-marker))
@@ -340,13 +403,13 @@ Don't change without re-evaluating the file.")
   "Comment a rule around point.
 With a prefix argument N, comment that many rules."
   (interactive "p")
-  (cg--comment/uncomment-rule 'comment n))
+  (cg--comment-uncomment-rule 'comment n))
 
 (defun cg-uncomment-rule (&optional n)
   "Uncomment a rule around point.
 With a prefix argument N, uncomment that many rules."
   (interactive "p")
-  (cg--comment/uncomment-rule nil n))
+  (cg--comment-uncomment-rule nil n))
 
 (defun cg-comment-or-uncomment-rule (&optional n)
   "Comment the rule at point.
@@ -363,6 +426,7 @@ With a prefix argument N, (un)comment that many rules."
       (cg-uncomment-rule n)
     (cg-comment-rule n)))
 
+
 
 ;;;###autoload
 (define-derived-mode cg-mode prog-mode "CG"
@@ -382,6 +446,7 @@ CG-mode provides the following specific keyboard key bindings:
          nil ;	  beginning-of-line		; SYNTAX-BEGIN
          (font-lock-syntactic-keywords . cg-font-lock-syntactic-keywords)
          (font-lock-syntactic-face-function . cg-font-lock-syntactic-face-function)))
+  (setq-local tool-bar-map cg-mode-tool-bar-map)
   ;; Indentation
   (set (make-local-variable 'indent-line-function) #'cg-indent-line)
   ;; Comments and blocks
@@ -393,12 +458,13 @@ CG-mode provides the following specific keyboard key bindings:
   (set (make-local-variable 'defun-prompt-regexp) (concat cg-kw-re "\\(?::[^\n\t ]+\\)[\t ]"))
   (set (make-local-variable 'beginning-of-defun-function) #'cg-beginning-of-defun)
   (set (make-local-variable 'end-of-defun-function) #'cg-end-of-defun)
-
+  ;; Completion:
+  (add-to-list 'completion-at-point-functions 'cg-complete-list-set)
+  ;; Syntax highlighting:
   (when font-lock-mode
     (setq font-lock-set-defaults nil)
     (font-lock-set-defaults)
-    ;; TODO: emacs 25 prefers `font-lock-ensure' and `font-lock-flush' over fontify
-    (font-lock-fontify-buffer))
+    (font-lock-ensure))
   (add-hook 'after-change-functions #'cg-after-change nil 'buffer-local)
   (let* ((buf (current-buffer))
          (hl-timer (run-with-idle-timer 1 'repeat 'cg-output-hl buf)))
@@ -525,6 +591,7 @@ From http://www.emacswiki.org/emacs/StringPermutations"
                input)))
 
 (defun cg-read-arg (prompt history &optional default)
+  "Ensure DEFAULT is in HISTORY and call `read-from-minibuffer' with PROMPT."
   (let* ((default (or default (car history)))
          (input
           (read-from-minibuffer
@@ -542,10 +609,11 @@ From http://www.emacswiki.org/emacs/StringPermutations"
       input)))
 
 (defun cg-occur-list (&optional prefix words)
-  "Do an occur-check for the left-side of a LIST/SET
-assignment. `words' is a space-separated list of words which
-*all* must occur between LIST/SET and =. Optional prefix argument
-`prefix' lets you specify a prefix to the name of LIST/SET.
+  "Do an occur-check for the left-side of a LIST/SET assignment.
+
+WORDS is a space-separated list of words which *all* must occur
+between LIST/SET and =.  Optional prefix argument PREFIX lets you
+specify a prefix to the name of LIST/SET.
 
 This is useful if you have a whole bunch of this stuff:
 LIST subst-mask/fem = (n m) (np m) (n f) (np f) ;
@@ -600,7 +668,102 @@ select the whole string \"SELECT:1022:rulename\")."
 
 
 
-;;; "Flycheck" ----------------------------------------------------------------
+;;; Flymake ----------------------------------------------------------------
+
+(defvar-local cg--flymake-proc nil)
+
+(defvar cg--flymake-ignore-unused-sets
+  (list "_MARK_" "_PAREN_" "_ENCL_" "_TARGET_" "_ATTACHTO_" "_SAME_BASIC_"))
+
+(defvar cg--flymake-warnings-pattern
+  (rx
+   (or
+    ;; apertium-nob.nob.rlx: Error: Garbage data encountered on line 7922 near `REMUVE`!
+    (seq ": " (group-n 1 (or "Error" "Warning")) ": "
+         (group-n 2 (* not-newline))
+         " on line " (group-n 3 (+ digit)))
+    ;; Unused sets:
+    ;; Line 0 set _TARGET_
+    ;; Line 103 set %én%
+    (seq bol "Line " (group-n 4 (+ digit))
+         " set " (group-n 5 (* not-newline)))
+    ;; apertium-nob.nob.rlx: Warning: LIST <noen> was defined twice with the same contents: Lines 375 and 33650.
+    (seq ": Warning: "
+         (group-n 6 (* not-newline))
+         ": Lines "
+         (group-n 7 (+ digit))
+         " and "
+         (group-n 8 (+ digit))))))
+
+(defun cg-flymake (report-fn &rest _args)
+  "Flymake backend for Constraint Grammar.
+REPORT-FN as in other flymake backends."
+  (unless (executable-find "vislcg3")
+    (error "Cannot find the `vislcg3' executable"))
+  (when (process-live-p cg--flymake-proc)
+    (kill-process cg--flymake-proc))
+  (save-restriction
+    (widen)
+    (let ((source (current-buffer))
+          (tempfile (with-temp-message ""
+                      (make-temp-file "cg-flymake" nil ".cg3" (buffer-string)))))
+      (setq
+       cg--flymake-proc
+       (make-process
+        :name "cg-flymake"
+        :noquery t
+        :connection-type 'pipe
+        :stderr nil           ; 2>&1
+        :buffer (generate-new-buffer " *cg-flymake*")
+        :command (list "vislcg3" "--grammar-only" "--show-unused-sets" "--grammar" tempfile)
+        :sentinel
+        (lambda (proc _event)
+          (when (eq 'exit (process-status proc))
+            (unwind-protect
+                (if (with-current-buffer source (eq proc cg--flymake-proc)) ; proc is not obsolete
+                    (with-current-buffer (process-buffer proc)
+                      (goto-char (point-min))
+                      (cl-loop
+                       while (search-forward-regexp
+                              cg--flymake-warnings-pattern ; either match groups (1 2 3) or (4 5) or (6 7 8)
+                              nil
+                              'noerror)
+                       unless (member (match-string 5) cg--flymake-ignore-unused-sets)
+                       for msg = (or (match-string 2)
+                                     (and (match-string 5)
+                                          (concat "Unused set " (match-string 5)))
+                                     (and (match-string 6)
+                                          (concat (match-string 6) " line " (match-string 8))))
+                       for (beg . end) = (flymake-diag-region
+                                          source
+                                          (string-to-number (or (match-string 3)
+                                                                (match-string 4)
+                                                                (match-string 7))))
+                       for type = (if (equal (match-string 1) "Error")
+                                      :error
+                                    :warning)
+                       collect (flymake-make-diagnostic source
+                                                        beg
+                                                        end
+                                                        type
+                                                        msg)
+                       into diags
+                       finally (funcall report-fn diags)))
+                  (flymake-log :warning "Canceling obsolete check %s"
+                               proc))
+              (kill-buffer (process-buffer proc))
+              (delete-file tempfile)))))))))
+
+(defun cg-flymake-setup-backend ()
+  "Enable flymake checking for Constraint Grammar buffers."
+  (add-hook 'flymake-diagnostic-functions 'cg-flymake nil t)
+  (flymake-mode 1))
+
+(add-hook 'cg-mode-hook 'cg-flymake-setup-backend)
+
+
+
+;;; Running examples ----------------------------------------------------------------
 (require 'compile)
 
 (defvar cg--file nil
@@ -611,23 +774,16 @@ buffer corresponds to.")
   "Which CG input buffer the `cg-mode' buffer corresponds to.")
 (make-variable-buffer-local 'cg--input-buffer)
 (defvar cg--tmp nil     ; TODO: could use cg--file iff buffer-modified-p
-  "Which temporary file was sent in lieu of `cg--file' to
-compilation (in case the buffer of `cg--file' was not saved)")
+  "Which temporary file was sent in lieu of `cg--file' to compilation.
+In case the buffer of `cg--file' was not saved.")
 (make-variable-buffer-local 'cg--tmp)
 (defvar cg--cache-in nil
-  "Which input buffer the `cg--check-cache-buffer' corresponds
-to.")
+  "Which input buffer the `cg--check-cache-buffer' corresponds to.")
 (make-variable-buffer-local 'cg--cache-in)
 (defvar cg--cache-pre-pipe nil
   "Which pre-pipe the output of `cg--check-cache-buffer' had.")
 (make-variable-buffer-local 'cg--cache-pre-pipe)
 
-(unless (fboundp 'file-name-base)	; shim for 24.3 function
-  (defun file-name-base (&optional filename)
-    (let ((filename (or filename (buffer-file-name))))
-      (file-name-nondirectory (file-name-sans-extension filename)))))
-
-
 (defun cg-edit-input (&optional pick-buffer)
   "Open a buffer to edit the input sent when running `cg-check'.
 With prefix argument PICK-BUFFER, prompt for a buffer (e.g. a
@@ -641,16 +797,17 @@ text file you've already opened) to use as CG input buffer."
 
 ;;;###autoload
 (defcustom cg-check-do-cache t
-  "If non-nil, `cg-check' caches the output of `cg-pre-pipe' (the
-cache is emptied whenever you make a change in the input buffer,
-or call `cg-check' from another CG file)."
+  "If non-nil, `cg-check' caches the output of `cg-pre-pipe'.
+The cache is emptied whenever you make a change in the input buffer,
+or call `cg-check' from another CG file."
   :group 'cg
   :type 'bool)
 
 (defvar cg--check-cache-buffer nil "See `cg-check-do-cache'.")
 
 (defun cg-input-mode-bork-cache (_from _to _len)
-  "Since `cg-check' will not reuse a cache unless `cg--file' and
+  "Purge the `cg-check' cache.
+Since `cg-check' will not reuse a cache unless `cg--file' and
 `cg--cache-in' match."
   (when cg--check-cache-buffer
     (with-current-buffer cg--check-cache-buffer
@@ -659,6 +816,7 @@ or call `cg-check' from another CG file)."
             cg--cache-in nil))))
 
 (defun cg-pristine-cache-buffer (file in pre-pipe)
+  "Make a new cache-buffer bound to this cg FILE, PRE-PIPE and input buffer IN."
   (with-current-buffer (setq cg--check-cache-buffer
                              (get-buffer-create "*cg-pre-cache*"))
     (widen)
@@ -674,6 +832,7 @@ or call `cg-check' from another CG file)."
 (define-derived-mode cg-input-mode fundamental-mode "CG-in"
   "Input for `cg-mode' buffers."
   (use-local-map cg-input-mode-map)
+  (setq-local tool-bar-map cg-input-mode-tool-bar-map)
   (add-hook 'after-change-functions #'cg-input-mode-bork-cache nil t))
 
 
@@ -714,6 +873,7 @@ after FILE; if it is 'pipe, the buffer will be named after the
     buf))
 
 (defun cg-get-file ()
+  "Get the name of the CG rule file for the output buffer."
   (list cg--file))
 
 (defconst cg-output-regexp-alist
@@ -780,25 +940,24 @@ This gets tacked on the end of the generated expressions.")
   "Used to turn ellipses into spaces when hiding analyses.")
 
 (defvar cg-sent-tag "\\bsent\\b"
-  "When using `cg-output-hide-analyses', any cohort matching this
-regex gets a newline tacked on after the wordform.")
+  "Any cohort matching this regex gets a newline tacked on after the wordform.
+For `cg-output-hide-analyses'.")
 
 (defvar cg-output-unhide-regex nil
-  "Regular expression exempt from hiding when doing
-`cg-output-hide-analyses'.")
+  "Regular expression exempt from hiding when doing `cg-output-hide-analyses'.")
 
 (defvar cg--output-hiding-analyses nil
-  "If non-nil, re-hide analyses after `cg-check'. Saves you from
-having to re-enter the buffer and press `h' if you you want to
-keep analyses hidden most of the time.")
+  "If non-nil, re-hide analyses after `cg-check'.
+Saves you from having to re-enter the buffer and press `h' if you
+you want to keep analyses hidden most of the time.")
 
 (defvar cg--output-unhide-history nil)
 
 
 
 (define-compilation-mode cg-output-mode "CG-out"
-  "Major mode for output of Constraint Grammar compilations and
-runs."
+  "Major mode for output of Constraint Grammar compilations and runs."
+  (setq-local tool-bar-map cg-output-mode-tool-bar-map)
   ;; cg-output-mode-font-lock-keywords applied automagically
   (set (make-local-variable 'compilation-skip-threshold)
        1)
@@ -837,9 +996,11 @@ runs."
 
 
 (defun cg-output-remove-overlay (overlay)
+  "Remove the invisible cg-output OVERLAY."
   (remove-overlays (overlay-start overlay) (overlay-end overlay) 'invisible 'cg-output))
 
 (defun cg-output-hide-region (from to)
+  "Hide the region FROM .. TO using invisible overlays."
   (remove-overlays from to 'invisible 'cg-output)
   (let ((o (make-overlay from to nil)))
     (overlay-put o 'evaporate t)
@@ -886,6 +1047,7 @@ from hiding.  Call `cg-output-show-all' to turn off all hiding."
       (cg-output-unhide-some cg-output-unhide-regex))))
 
 (defun cg-output-unhide-some (needle)
+  "Remove invisible-overlays matching NEEDLE."
   (save-excursion
     (goto-char (point-min))
     (while (re-search-forward needle nil 'noerror)
@@ -936,6 +1098,7 @@ Use 0 to check immediately after each change."
 
 (defvar cg--after-change-timer nil)
 (defun cg-after-change (_from _to _len)
+  "For use in `after-change-functions'."
   (when (and cg-check-after-change
              (not (member cg--after-change-timer timer-list)))
     (setq
@@ -945,37 +1108,58 @@ Use 0 to check immediately after each change."
       nil
       (lambda ()
         (unless (cg-output-running)
-	  (with-demoted-errors (cg-check))))))))
+	  (with-demoted-errors "cg-after-change: %S" (cg-check))))))))
+
+(defcustom cg-output-always-highlight nil
+  "Regexp patterns of tags to always highlight in output, with a face to use."
+  :local t
+  :type '(repeat (cons regexp face)))
+
+(defun cg-output-hl-always-patterns ()
+  "Highlight the `cg-output-always-highlight' patterns in output.
+That variable is buffer-local, but the value is copied from cg
+buffer to cg-output-buffer on check."
+  (with-current-buffer (cg-output-buffer)
+    (dolist (pat.face cg-output-always-highlight)
+      (cg-hl-pat (car pat.face) (cdr pat.face)))))
 
 (defun cg-output-hl (cg-buffer)
-  "Highlight the symbol at point in the output buffer."
+  "Highlight the symbol at point of CG-BUFFER in the output buffer."
   (when (eq (current-buffer) cg-buffer)
     (let* ((sym (symbol-at-point))
-	   (sym-re (concat "\\(?:^\\|[ \"(:]\\)\\("
-			   (regexp-quote (symbol-name sym))
-			   "\\)\\(?:[:)\" ]\\|$\\)")))
+	   (sym-re (regexp-quote (symbol-name sym))))
       ;; TODO: make regexp-opts of the LIST definitions and search
       ;; those as well?
       (with-current-buffer (cg-output-buffer)
-	(when (and sym
-		   (get-buffer-window)
-		   (not (cg-output-running)))
-	  (remove-overlays (point-min) (point-max) 'face cg-output-highlight-face)
-	  (goto-char (point-min))
-	  (while (re-search-forward sym-re nil 'noerror)
-	    (overlay-put (make-overlay (match-beginning 1) (match-end 1))
-			 'face cg-output-highlight-face)))))))
+        (when (and sym-re
+	           (get-buffer-window)
+	           (not (cg-output-running)))
+          (remove-overlays (point-min) (point-max) 'face cg-output-highlight-face)
+          (cg-hl-pat sym-re cg-output-highlight-face))))))
+
+(defun cg-hl-pat (pat face)
+  "Highlight tag matching regex PAT with FACE in current buffer."
+  (let ((pat-delimited (concat "\\(?:^\\|[ \"(:]\\)\\("
+			       pat
+                               "\\)\\(?:[:)\" ]\\|$\\)")))
+    (goto-char (point-min))
+    (while (re-search-forward pat-delimited nil 'noerror)
+      (overlay-put (make-overlay (match-beginning 1) (match-end 1))
+		   'face face))))
 
 (defun cg-output-running ()
+  "Check if we're already running a vislcg3 process."
   (let ((proc (get-buffer-process (cg-output-buffer))))
     (and proc (eq (process-status proc) 'run))))
 
 (defun cg-output-buffer-name (mode)
+  "Construct a name for the output buffer if we're in the right MODE."
   (if (equal mode "cg-output")
       (concat "*CG output for " (file-name-base cg--file) "*")
     (error "Unexpected mode %S" mode)))
 
 (defun cg-output-buffer ()
+  "Get or make the output buffer for this grammar file."
   (let ((cg--file (if (eq major-mode 'cg-mode)
 		      (buffer-file-name)
 		    cg--file)))
@@ -992,8 +1176,9 @@ Use 0 to check immediately after each change."
   (process-send-eof proc))
 
 (defun cg-check ()
-  "Run vislcg3 --trace on the buffer (a temporary file is created
-in case you haven't saved yet).
+  "Run vislcg3 --trace on the buffer with your example inputs.
+
+A temporary file is created in case you haven't saved yet.
 
 If you've set `cg-pre-pipe', input will first be sent through
 that.  Set your test input sentence(s) with `cg-edit-input'.
@@ -1022,9 +1207,12 @@ Similarly, `cg-post-pipe' is run on output."
                    (compilation-start
                     cmd
                     'cg-output-mode
-                    'cg-output-buffer-name))))
+                    'cg-output-buffer-name)))
+       ; copy of the buffer-local variable
+       (patterns cg-output-always-highlight))
 
     (with-current-buffer out
+      (setq-local cg-output-always-highlight patterns)
       (setq cg--tmp tmp)
       (setq cg--file file))
 
@@ -1057,6 +1245,7 @@ Similarly, `cg-post-pipe' is run on output."
     (display-buffer out)))
 
 (defun cg-check-finish-function (buffer _change)
+  "BUFFER as in `compilation-finish-functions'."
   ;; Note: this makes `recompile' not work, which is why `g' is
   ;; rebound in `cg-output-mode'
   (let ((w (get-buffer-window buffer)))
@@ -1065,32 +1254,38 @@ Similarly, `cg-post-pipe' is run on output."
         (scroll-up-line 4))))
   (with-current-buffer buffer
     (delete-file cg--tmp))
+  (cg-output-hl-always-patterns)
   (when cg--output-hiding-analyses
     (cg-output-hide-analyses)))
 
 (defun cg-back-to-file-and-edit-input ()
+  "Open buffer with example inputs."
   (interactive)
   (cg-back-to-file)
   (cg-edit-input))
 
 (defun cg-back-to-file ()
+  "Open buffer of grammar rules file."
   (interactive)
   (let ((cg-buffer (find-buffer-visiting cg--file)))
-    (bury-buffer)
+    (if (cdr (window-list))
+        (delete-window)
+      (bury-buffer))
     (let ((cg-window (get-buffer-window cg-buffer)))
-
       (if cg-window
 	  (select-window cg-window)
 	(pop-to-buffer cg-buffer)))))
 
 
 (defun cg-back-to-file-and-check ()
+  "Open buffer of grammar rules file and check examples."
   (interactive)
   (cg-back-to-file)
   (cg-check))
 
 
 (defun cg-toggle-check-after-change ()
+  "Toggle whether we should run examples on each change to rules."
   (interactive)
   (setq cg-check-after-change (not cg-check-after-change))
   (message "%s after each change" (if cg-check-after-change
@@ -1102,56 +1297,72 @@ Similarly, `cg-post-pipe' is run on output."
 
 ;;; xref support --------------------------------------------------------------
 
-(when (featurep 'xref)
-  (declare-function xref-make-bogus-location "xref" (message))
-  (declare-function xref-make "xref" (summary location))
-  (declare-function xref-collect-references "xref" (symbol dir))
+(declare-function xref-make-bogus-location "xref" (message))
+(declare-function xref-make "xref" (summary location))
+(declare-function xref-collect-references "xref" (symbol dir))
 
-  (cl-defmethod xref-backend-identifier-at-point ((_backend (eql cg)))
-    (format "%s" (symbol-at-point)))
+(cl-defmethod xref-backend-identifier-at-point ((_backend (eql cg)))
+  "Return identifier at point for CG syntax."
+  (format "%s" (symbol-at-point)))
 
-  (defvar cg--set-definition-re "^ *\\(?:[Ll][Ii][Ss]\\|[Ss][Ee]\\)t\\s +\\(%s\\)\\(?:\\s \\|=\\)")
+(defvar cg--set-definition-re "^ *\\(?:[Ll][Ii][Ss]\\|[Ss][Ee]\\)t\\s +\\(%s\\)\\(?:\\s \\|=\\)")
 
-  (cl-defmethod xref-backend-identifier-completion-table ((_backend (eql cg)))
-    (completion-table-dynamic
-     (lambda (prefix)
-       (let* ((prefix-re (concat (replace-quote prefix)
-                                 "\\S *"))
-              (def-re (format cg--set-definition-re prefix-re))
-              matches)
-         (save-excursion
-           (save-restriction
-             (widen)
-             (goto-char (point-min))
-             (while (re-search-forward def-re nil 'noerror)
-               (push (match-string-no-properties 1) matches))))
-         matches))
-     'switch-buffer))
+(cl-defmethod xref-backend-identifier-completion-table ((_backend (eql cg)))
+  "Return the completion table for CG identifiers."
+  (completion-table-dynamic
+   (lambda (prefix)
+     (let* ((prefix-re (concat (replace-quote prefix)
+                               "\\S *"))
+            (def-re (format cg--set-definition-re prefix-re))
+            matches)
+       (save-excursion
+         (save-restriction
+           (widen)
+           (goto-char (point-min))
+           (while (re-search-forward def-re nil 'noerror)
+             (push (match-string-no-properties 1) matches))))
+       matches))
+   'switch-buffer))
 
-  (cl-defmethod xref-backend-definitions ((_backend (eql cg)) symbol)
-    (let* ((loc
-            (save-excursion
-              (save-restriction
-                (widen)
-                (goto-char (point-min))
-                (and
-                 (re-search-forward (format cg--set-definition-re symbol) nil 'noerror)
-                 (xref-make-file-location (buffer-file-name)
-                                          (line-number-at-pos) ; TODO: this is slow!
-                                          (current-column)))))))
-      (when loc
-        (list (xref-make (format "%s" symbol) loc)))))
+(defun cg-complete-list-set ()
+  "Simple dabbrev-like completion for `completion-at-point-functions'."
+  (let ((bounds (or (bounds-of-thing-at-point 'symbol)
+                    (cons (point) (point)))))
+    (list (car bounds)
+          (cdr bounds)
+          (xref-backend-identifier-completion-table 'cg))))
 
-  (cl-defmethod xref-backend-references ((_backend (eql cg)) symbol)
-    (message "Not yet implemented")
-    nil)
+(cl-defmethod xref-backend-definitions ((_backend (eql cg)) symbol)
+  "Find definitions of SYMBOL.
+Only lists/sets for now."
+  (let* ((loc
+          (save-excursion
+            (save-restriction
+              (widen)
+              (goto-char (point-min))
+              (and
+               (re-search-forward (format cg--set-definition-re symbol) nil 'noerror)
+               (xref-make-file-location (buffer-file-name)
+                                        (line-number-at-pos) ; TODO: this is slow!
+                                        (current-column)))))))
+    (when loc
+      (list (xref-make (format "%s" symbol) loc)))))
 
-  (defun cg--xref-backend () 'cg)
-  (add-hook 'cg-mode-hook
-            (defun cg--setup-xref ()
-              (define-key cg-mode-map (kbd "M-.") 'xref-find-definitions)
-              (add-hook 'xref-backend-functions #'cg--xref-backend nil t))))
+(cl-defmethod xref-backend-references ((_backend (eql cg)) symbol)
+  "Find references of SYMBOL – not yet implemented."
+  (message "Not yet implemented")
+  nil)
 
+(defun cg--xref-backend ()
+  "Use this in `xref-backend-functions' to get CG references."
+  'cg)
+
+(defun cg-setup-xref ()
+  "Use this in `cg-mode-hook' to get CG cross-references."
+  (define-key cg-mode-map (kbd "M-.") 'xref-find-definitions)
+  (add-hook 'xref-backend-functions #'cg--xref-backend nil t))
+
+(add-hook 'cg-mode-hook #'cg-setup-xref)
 
 ;;; Keybindings ---------------------------------------------------------------
 (define-key cg-mode-map (kbd "C-c C-o") #'cg-occur-list)
@@ -1181,8 +1392,9 @@ Similarly, `cg-post-pipe' is run on output."
 (define-key cg-mode-map (kbd "C-c C-n") 'next-error)
 (define-key cg-mode-map (kbd "C-c C-p") 'previous-error)
 
-;;; Turn on for .cg3 files ----------------------------------------------------
+;;; Turn on for .cg3 and .rlx files -------------------------------------------
 ;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.rlx\\'" . cg-mode))
 (add-to-list 'auto-mode-alist '("\\.cg3\\'" . cg-mode))
 ;; Tino Didriksen recommends this file suffix.
 
