@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -34,10 +34,113 @@ constexpr UChar esc_lt = '\1';
 ApertiumApplicator::ApertiumApplicator(std::ostream& ux_err)
   : GrammarApplicator(ux_err)
 {
-	wordform_case = false;
-	unique_tags = false;
-	print_word_forms = true;
-	print_only_first = false;
+}
+
+void ApertiumApplicator::parseStreamVar(const SingleWindow* cSWindow, UString& cleaned,
+					uint32FlatHashMap& variables_set, uint32FlatHashSet& variables_rem, uint32SortedVector& variables_output) {
+	size_t packoff = 0;
+	if (u_strncmp(&cleaned[0], STR_CMD_SETVAR.data(), STR_CMD_SETVAR.size()) == 0) {
+		// u_fprintf(ux_stderr, "Info: SETVAR encountered on line %u.\n", numLines);
+		cleaned[packoff - 1] = 0;
+		// line[0] = 0;
+
+		UChar* s = &cleaned[STR_CMD_SETVAR.size()];
+		UChar* c = u_strchr(s, ',');
+		UChar* d = u_strchr(s, '=');
+		if (c == 0 && d == 0) {
+			Tag* tag = addTag(s);
+			variables_set[tag->hash] = grammar->tag_any;
+			variables_rem.erase(tag->hash);
+			variables_output.insert(tag->hash);
+			if (cSWindow == nullptr) {
+				variables[tag->hash] = grammar->tag_any;
+			}
+		}
+		else {
+			uint32_t a = 0, b = 0;
+			while (c || d) {
+				if (d && (d < c || c == nullptr)) {
+					d[0] = 0;
+					if (!s[0]) {
+						u_fprintf(ux_stderr, "Warning: SETVAR on line %u had no identifier before the =! Defaulting to identifier *.\n", numLines);
+						a = grammar->tag_any;
+					}
+					else {
+						a = addTag(s)->hash;
+					}
+					if (c) {
+						c[0] = 0;
+						s = c + 1;
+					}
+					if (!d[1]) {
+						u_fprintf(ux_stderr, "Warning: SETVAR on line %u had no value after the =! Defaulting to value *.\n", numLines);
+						b = grammar->tag_any;
+					}
+					else {
+						b = addTag(d + 1)->hash;
+					}
+					if (!c) {
+						d = nullptr;
+						s = nullptr;
+					}
+					variables_set[a] = b;
+					variables_rem.erase(a);
+					variables_output.insert(a);
+				}
+				else if (c && (c < d || d == nullptr)) {
+					c[0] = 0;
+					if (!s[0]) {
+						u_fprintf(ux_stderr, "Warning: SETVAR on line %u had no identifier after the ,! Defaulting to identifier *.\n", numLines);
+						a = grammar->tag_any;
+					}
+					else {
+						a = addTag(s)->hash;
+					}
+					s = c + 1;
+					variables_set[a] = grammar->tag_any;
+					variables_rem.erase(a);
+					variables_output.insert(a);
+				}
+				if (s) {
+					c = u_strchr(s, ',');
+					d = u_strchr(s, '=');
+					if (c == nullptr && d == nullptr) {
+						a = addTag(s)->hash;
+						variables_set[a] = grammar->tag_any;
+						variables_rem.erase(a);
+						variables_output.insert(a);
+						s = nullptr;
+					}
+				}
+			}
+		}
+	}
+	else if (u_strncmp(&cleaned[0], STR_CMD_REMVAR.data(), STR_CMD_REMVAR.size()) == 0) {
+		//u_fprintf(ux_stderr, "Info: REMVAR encountered on line %u.\n", numLines);
+		cleaned[packoff - 1] = 0;
+		// line[0] = 0;
+
+		UChar* s = &cleaned[STR_CMD_REMVAR.size()];
+		UChar* c = u_strchr(s, ',');
+		uint32_t a = 0;
+		while (c && *c) {
+			c[0] = 0;
+			if (s[0]) {
+				a = addTag(s)->hash;
+				variables_set.erase(a);
+				variables_rem.insert(a);
+				variables_output.insert(a);
+			}
+			s = c + 1;
+			c = u_strchr(s, ',');
+		}
+		if (s && s[0]) {
+			a = addTag(s)->hash;
+			variables_set.erase(a);
+			variables_rem.insert(a);
+			variables_output.insert(a);
+		}
+	}
 }
 
 /*
@@ -86,11 +189,11 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 
 	uint32_t resetAfter = ((num_windows + 4) * 2 + 1);
 
-	const auto wb_start = "[["_us;
-	const auto wb_end = "]]"_us;
+	constexpr UStringView wb_start{ u"[[" };
+	constexpr UStringView wb_end{ u"]]" };
 
-	begintag = addTag(stringbits[S_BEGINTAG])->hash; // Beginning of sentence tag
-	endtag = addTag(stringbits[S_ENDTAG])->hash;     // End of sentence tag
+	begintag = addTag(STR_BEGINTAG)->hash; // Beginning of sentence tag
+	endtag = addTag(STR_ENDTAG)->hash;     // End of sentence tag
 
 	SingleWindow* cSWindow = nullptr; // Current single window (Cohort frame)
 	Cohort* cCohort = nullptr;        // Current cohort
@@ -102,6 +205,10 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 	gtimer = getticks();
 	ticks timer(gtimer);
 
+	uint32FlatHashMap variables_set;
+	uint32FlatHashSet variables_rem;
+	uint32SortedVector variables_output;
+
 	ux_stripBOM(input);
 
 	auto ensure_endtag = [&]() {
@@ -115,6 +222,11 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 	auto flush = [&](bool n = false) {
 		ensure_endtag();
 
+		auto backSWindow = n ? gWindow->back() : nullptr;
+		if (backSWindow) {
+			backSWindow->flush_after = true;
+		}
+
 		if (!blank.empty()) {
 			if (lCohort) {
 				lCohort->text += blank;
@@ -126,7 +238,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 				lSWindow->text += blank;
 			}
 			else {
-				u_fprintf(output, "%S", blank.c_str());
+				u_fprintf(output, "%S", blank.data());
 			}
 			blank.clear();
 		}
@@ -149,7 +261,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 			u_fprintf(output, "%C", c); // eg. final newline
 		}
 
-		if (n) {
+		if (n && !backSWindow) {
 			u_fputc('\0', output);
 		}
 		u_fflush(output);
@@ -162,6 +274,10 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 		cSWindow = nullptr;
 		cCohort = nullptr;
 		token.clear();
+		variables_rem.clear();
+		variables_set.clear();
+		variables_output.clear();
+		variables.clear();
 	};
 
 	while ((c = u_fgetc(input)) != U_EOF) {
@@ -209,6 +325,12 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 		}
 		else if (in_blank && c == ']') {
 			in_blank = false;
+			if (blank.size() > 14 // contains at least "[<STREAMCMD:>]"
+			    && blank[1] == '<' && blank[blank.size() - 2] == '>') {
+				UString cleaned = blank.substr(1, blank.size() - 3);
+				parseStreamVar(cSWindow, cleaned,
+					       variables_set, variables_rem, variables_output);
+			}
 		}
 		else if (!in_blank && c == '$') {
 			if (!in_cohort) {
@@ -222,13 +344,16 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 				auto b = blank.find(wb_start);
 				auto e = blank.find(wb_end, b);
 				if (b != UString::npos && e != UString::npos) {
-					// Word-bound blanks are always immediately prior to the token they belong to
-					wblank = blank.substr(b);
-					blank.erase(b, UString::npos);
+					// Word-bound blanks are always immediately prior to the token they belong to,
+					// except closing word blanks which must be treated as normal blanks
+					if (e != b + 3 || blank[b + 2] != '/') {
+						wblank = blank.substr(b);
+						blank.erase(b, UString::npos);
+					}
 				}
 			}
 			if (!wblank.empty() && (wblank[wblank.size() - 1] != ']' || wblank[wblank.size() - 2] != ']')) {
-				u_fprintf(ux_stderr, "Error: Word-bound blank was not immediately prior to token on line %u.\n", numLines);
+				u_fprintf(ux_stderr, "Error: Word-bound blank was not immediately prior to token on line %u\n", numLines);
 				CG3Quit(1);
 			}
 
@@ -251,19 +376,14 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 
 				cSWindow = gWindow->allocAppendSingleWindow();
 
-				// Create 0th Cohort which serves as the beginning of sentence
-				auto cCohort = alloc_cohort(cSWindow);
-				cCohort->global_number = gWindow->cohort_counter++;
-				cCohort->wordform = tag_begin;
+				initEmptySingleWindow(cSWindow);
 
-				auto cReading = alloc_reading(cCohort);
-				cReading->baseform = begintag;
-				insert_if_exists(cReading->parent->possible_sets, grammar->sets_any);
-				addTagToReading(*cReading, begintag);
-
-				cCohort->appendReading(cReading);
-
-				cSWindow->appendCohort(cCohort);
+				cSWindow->variables_set = variables_set;
+				variables_set.clear();
+				cSWindow->variables_rem = variables_rem;
+				variables_rem.clear();
+				cSWindow->variables_output = variables_output;
+				variables_output.clear();
 
 				lSWindow = cSWindow;
 				lSWindow->text = blank;
@@ -312,7 +432,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 					if (*p == '>') {
 						Tag* t = addTag(blank);
 						addTagToReading(*cCohort->wread, t);
-						//u_fprintf(ux_stderr, "Adding tag %S\n", tag.c_str());
+						//u_fprintf(ux_stderr, "Adding tag %S\n", tag.data());
 						blank.clear();
 						continue;
 					}
@@ -369,6 +489,9 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 			}
 			insert_if_exists(cCohort->possible_sets, grammar->sets_any);
 			cSWindow->appendCohort(cCohort);
+			if (cCohort->wordform->tag[2] == '@') {
+				cCohort->type |= CT_AP_UNKNOWN;
+			}
 
 			bool did_delim = false;
 			if (cCohort && cSWindow->cohorts.size() >= soft_limit && grammar->soft_delimiters && doesSetMatchCohortNormal(*cCohort, grammar->soft_delimiters->number)) {
@@ -383,7 +506,7 @@ void ApertiumApplicator::runGrammarOnText(std::istream& input, std::ostream& out
 			} // end >= soft_limit
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.data(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -493,7 +616,7 @@ void ApertiumApplicator::processReading(Reading* cReading, UChar* p, Tag* wform)
 		}
 	}
 
-	if (bf.size() > 1) {
+	if (!bf.empty() || !tags.empty() || !prefix_tags.empty()) {
 		bf += '"';
 		taglist.push_back(addTag(bf));
 		taglist.insert(taglist.end(), tags.begin(), tags.end());
@@ -514,7 +637,7 @@ void ApertiumApplicator::processReading(Reading* cReading, UChar* p, Tag* wform)
 				}
 				// Add tags
 				TagList mappings;
-				TagVector::iterator iter = riter.base();
+				auto iter = riter.base();
 				for (--iter; iter != taglist.end(); ++iter) {
 					if ((*iter)->type & T_MAPPING || (*iter)->tag[0] == grammar->mapping_prefix) {
 						mappings.push_back(*iter);
@@ -572,7 +695,7 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 
 	if (reading->baseform) {
 		// Lop off the initial and final '"' characters
-		UnicodeString bf(single_tags[reading->baseform]->tag.c_str() + 1, static_cast<int32_t>(single_tags[reading->baseform]->tag.size() - 2));
+		UnicodeString bf(grammar->single_tags[reading->baseform]->tag.data() + 1, SI32(grammar->single_tags[reading->baseform]->tag.size() - 2));
 
 		if (wordform_case) {
 			if (casing == ApertiumCasing::Upper) {
@@ -591,11 +714,23 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 			if (bf[i] == '^' || bf[i] == '\\' || bf[i] == '/' || bf[i] == '$' || bf[i] == '[' || bf[i] == ']' || bf[i] == '{' || bf[i] == '}' || bf[i] == '<' || bf[i] == '>') {
 				bf_escaped += '\\';
 			}
+			if ((reading->parent->type & CT_AP_UNKNOWN) && bf[i] == '@') {
+				bf_escaped += '\\';
+			}
 			bf_escaped += bf[i];
 		}
-		u_fprintf(output, "%S", bf_escaped.c_str());
+		if(surface_readings && bf.length() > 0 && bf_escaped[0] == '@') {
+			bf_escaped[0] = '#';
+		}
+		u_fprintf(output, "%S", bf_escaped.data());
 
-		// Tag::printTagRaw(output, single_tags[reading->baseform]);
+		// Tag::printTagRaw(output, grammar->single_tags[reading->baseform]);
+	}
+
+	if(surface_readings && !trace) {
+		// As with lt-proc -g, when output has tags (meaning
+		// ungenerated words), don't print unless tracing (lt-proc -d)
+		return;
 	}
 
 	// Reorder: MAPPING tags should appear before the join of multiword tags,
@@ -605,7 +740,7 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 	Reading::tags_list_t multitags_list; // everything after a +, until the first MAPPING tag
 	bool multi = false;
 	for (auto tter : reading->tags_list) {
-		const Tag* tag = single_tags[tter];
+		const Tag* tag = grammar->single_tags[tter];
 		if (tag->tag[0] == '+') {
 			multi = true;
 		}
@@ -613,6 +748,10 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 			multi = false;
 		}
 
+		if (tag->type & T_DEPENDENCY && has_dep && !dep_original) {
+			continue;
+		}
+
 		if (multi) {
 			multitags_list.push_back(tter);
 		}
@@ -623,6 +762,7 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 	tags_list.insert(tags_list.end(), multitags_list.begin(), multitags_list.end());
 
 	uint32SortedVector used_tags;
+	UStringView escape{ surface_readings ? u"\\" : u"" };
 	for (auto tter : tags_list) {
 		if (unique_tags) {
 			if (used_tags.find(tter) != used_tags.end()) {
@@ -633,20 +773,39 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output, Ap
 		if (tter == endtag || tter == begintag) {
 			continue;
 		}
-		const Tag* tag = single_tags[tter];
+		const Tag* tag = grammar->single_tags[tter];
 		if (!(tag->type & T_BASEFORM) && !(tag->type & T_WORDFORM)) {
 			if (tag->tag[0] == '+') {
-				u_fprintf(output, "%S", tag->tag.c_str());
+				u_fprintf(output, "%S", tag->tag.data());
 			}
 			else if (tag->tag[0] == '&') {
-				u_fprintf(output, "<%S>", substr(tag->tag, 2).c_str());
+				u_fprintf(output, "%S<%S%S>", escape.data(), substr(tag->tag, 2).data(), escape.data());
 			}
 			else {
-				u_fprintf(output, "<%S>", tag->tag.c_str());
+				u_fprintf(output, "%S<%S%S>", escape.data(), tag->tag.data(), escape.data());
 			}
 		}
 	}
 
+	if (has_dep && !(reading->parent->type & CT_REMOVED) && !reading->next) {
+		if (!reading->parent->dep_self) {
+			reading->parent->dep_self = reading->parent->global_number;
+		}
+		const Cohort* pr = nullptr;
+		pr = reading->parent;
+		if (reading->parent->dep_parent != DEP_NO_PARENT) {
+			if (reading->parent->dep_parent == 0) {
+				pr = reading->parent->parent->cohorts[0];
+			}
+			else if (reading->parent->parent->parent->cohort_map.find(reading->parent->dep_parent) != reading->parent->parent->parent->cohort_map.end()) {
+				pr = reading->parent->parent->parent->cohort_map[reading->parent->dep_parent];
+			}
+		}
+
+		constexpr UChar pattern[] = { '<', '#', '%', 'u', u'\u2192', '%', 'u', '>', 0 };
+		u_fprintf_u(output, pattern, reading->parent->local_number, pr->local_number);
+	}
+
 	if (trace) {
 		for (auto iter_hb : reading->hit_by) {
 			u_fputc('<', output);
@@ -675,7 +834,7 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output) {
 		}
 		if (last->baseform) {
 			// Including the initial and final '"' characters
-			UString* bftag = &single_tags[last->baseform]->tag;
+			UString* bftag = &grammar->single_tags[last->baseform]->tag;
 			// Excluding the initial and final '"' characters
 			size_t bf_length = bftag->size() - 2;
 			UString* wftag = &reading->parent->wordform->tag;
@@ -717,11 +876,112 @@ void ApertiumApplicator::printReading(Reading* reading, std::ostream& output) {
 	printReading(reading, output, casing, firstlower);
 }
 
+void ApertiumApplicator::printCohort(Cohort* cohort, std::ostream& output) {
+	cohort->unignoreAll();
+
+	if (!split_mappings) {
+		mergeMappings(*cohort);
+	}
+
+	if (!cohort->wblank.empty()) {
+		u_fprintf(output, "%S", cohort->wblank.data());
+	}
+
+	// Start of cohort
+	if (delimit_lexical_units) {
+		u_fprintf(output, "^");
+	}
+
+	if (print_word_forms == true) {
+		// Lop off the initial and final '"' characters
+		// ToDo: A copy does not need to be made here - use pointer offsets
+		UnicodeString wf(cohort->wordform->tag.data() + 2, SI32(cohort->wordform->tag.size() - 4));
+		UString wf_escaped;
+		for (int i = 0; i < wf.length(); ++i) {
+			if (wf[i] == '^' || wf[i] == '\\' || wf[i] == '/' || wf[i] == '$' || wf[i] == '[' || wf[i] == ']' || wf[i] == '{' || wf[i] == '}' || wf[i] == '<' || wf[i] == '>') {
+				wf_escaped += '\\';
+			}
+			if ((cohort->type & CT_AP_UNKNOWN) && wf[i] == '@') {
+				wf_escaped += '\\';
+			}
+			wf_escaped += wf[i];
+		}
+		u_fprintf(output, "%S", wf_escaped.data());
+
+		// Print the static reading tags
+		if (cohort->wread) {
+			for (auto tter : cohort->wread->tags_list) {
+				if (tter == cohort->wordform->hash) {
+					continue;
+				}
+				const Tag* tag = grammar->single_tags[tter];
+				u_fprintf(output, "<%S>", tag->tag.data());
+			}
+		}
+	}
+
+	bool need_slash = print_word_forms;
+
+	//Tag::printTagRaw(output, grammar->single_tags[cohort->wordform]);
+	std::sort(cohort->readings.begin(), cohort->readings.end(), CG3::Reading::cmp_number);
+	for (auto reading : cohort->readings) {
+		if (need_slash) {
+			u_fprintf(output, "/");
+		}
+		need_slash = true;
+		if (grammar->sub_readings_ltr && reading->next) {
+			reading = reverse(reading);
+		}
+		printReading(reading, output);
+		if (print_only_first == true) {
+			break;
+		}
+	}
+
+	if (trace) {
+		std::sort(cohort->delayed.begin(), cohort->delayed.end(), CG3::Reading::cmp_number);
+		for (auto reading : cohort->delayed) {
+			if (need_slash) {
+				u_fprintf(output, "/%C", not_sign);
+			}
+			need_slash = true;
+			if (grammar->sub_readings_ltr && reading->next) {
+				reading = reverse(reading);
+			}
+			printReading(reading, output);
+		}
+		std::sort(cohort->deleted.begin(), cohort->deleted.end(), CG3::Reading::cmp_number);
+		for (auto reading : cohort->deleted) {
+			if (need_slash) {
+				u_fprintf(output, "/%C", not_sign);
+			}
+			need_slash = true;
+			if (grammar->sub_readings_ltr && reading->next) {
+				reading = reverse(reading);
+			}
+			printReading(reading, output);
+		}
+	}
+
+	if (delimit_lexical_units) {
+		u_fprintf(output, "$");
+	}
+	// End of cohort
+
+	if (!cohort->text.empty()) {
+		u_fprintf(output, "%S", cohort->text.data());
+	}
+	for (auto& c : cohort->removed) {
+		if (!c->text.empty()) {
+			u_fprintf(output, "%S", c->text.data());
+		}
+	}
+}
 
 void ApertiumApplicator::printSingleWindow(SingleWindow* window, std::ostream& output) {
 	// Window text comes at the left
 	if (!window->text.empty()) {
-		u_fprintf(output, "%S", window->text.c_str());
+		u_fprintf(output, "%S", window->text.data());
 	}
 
 	for (uint32_t c = 0; c < window->cohorts.size(); c++) {
@@ -730,112 +990,24 @@ void ApertiumApplicator::printSingleWindow(SingleWindow* window, std::ostream& o
 		if (c == 0) { // Skip magic cohort
 			for (auto& c : cohort->removed) {
 				if (!c->text.empty()) {
-					u_fprintf(output, "%S", c->text.c_str());
+					u_fprintf(output, "%S", c->text.data());
 				}
 			}
 			continue;
 		}
 
-		cohort->unignoreAll();
-
-		if (!split_mappings) {
-			mergeMappings(*cohort);
-		}
-
-		if (!cohort->wblank.empty()) {
-			u_fprintf(output, "%S", cohort->wblank.c_str());
-		}
-
-		// Start of cohort
-		u_fprintf(output, "^");
-
-		if (print_word_forms == true) {
-			// Lop off the initial and final '"' characters
-			// ToDo: A copy does not need to be made here - use pointer offsets
-			UnicodeString wf(cohort->wordform->tag.c_str() + 2, static_cast<int32_t>(cohort->wordform->tag.size() - 4));
-			UString wf_escaped;
-			for (int i = 0; i < wf.length(); ++i) {
-				if (wf[i] == '^' || wf[i] == '\\' || wf[i] == '/' || wf[i] == '$' || wf[i] == '[' || wf[i] == ']' || wf[i] == '{' || wf[i] == '}' || wf[i] == '<' || wf[i] == '>') {
-					wf_escaped += '\\';
-				}
-				wf_escaped += wf[i];
-			}
-			u_fprintf(output, "%S", wf_escaped.c_str());
-
-			// Print the static reading tags
-			if (cohort->wread) {
-				for (auto tter : cohort->wread->tags_list) {
-					if (tter == cohort->wordform->hash) {
-						continue;
-					}
-					const Tag* tag = single_tags[tter];
-					u_fprintf(output, "<%S>", tag->tag.c_str());
-				}
-			}
-		}
-
-		bool need_slash = print_word_forms;
-
-		//Tag::printTagRaw(output, single_tags[cohort->wordform]);
-		std::sort(cohort->readings.begin(), cohort->readings.end(), CG3::Reading::cmp_number);
-		for (auto reading : cohort->readings) {
-			if (need_slash) {
-				u_fprintf(output, "/");
-			}
-			need_slash = true;
-			if (grammar->sub_readings_ltr && reading->next) {
-				reading = reverse(reading);
-			}
-			printReading(reading, output);
-			if (print_only_first == true) {
-				break;
-			}
-		}
-
-		if (trace) {
-			std::sort(cohort->delayed.begin(), cohort->delayed.end(), CG3::Reading::cmp_number);
-			for (auto reading : cohort->delayed) {
-				if (need_slash) {
-					u_fprintf(output, "/%C", not_sign);
-				}
-				need_slash = true;
-				if (grammar->sub_readings_ltr && reading->next) {
-					reading = reverse(reading);
-				}
-				printReading(reading, output);
-			}
-			std::sort(cohort->deleted.begin(), cohort->deleted.end(), CG3::Reading::cmp_number);
-			for (auto reading : cohort->deleted) {
-				if (need_slash) {
-					u_fprintf(output, "/%C", not_sign);
-				}
-				need_slash = true;
-				if (grammar->sub_readings_ltr && reading->next) {
-					reading = reverse(reading);
-				}
-				printReading(reading, output);
-			}
-		}
-
-		u_fprintf(output, "$");
-		// End of cohort
-
-		if (!cohort->text.empty()) {
-			u_fprintf(output, "%S", cohort->text.c_str());
-		}
-		for (auto& c : cohort->removed) {
-			if (!c->text.empty()) {
-				u_fprintf(output, "%S", c->text.c_str());
-			}
-		}
-
+		printCohort(cohort, output);
 		u_fflush(output);
 	}
 
 	if (!window->text_post.empty()) {
-		u_fprintf(output, "%S", window->text_post.c_str());
+		u_fprintf(output, "%S", window->text_post.data());
 		u_fflush(output);
 	}
+
+	if (window->flush_after) {
+		u_fputc('\0', output);
+	}
 }
 
 void ApertiumApplicator::mergeMappings(Cohort& cohort) {
