@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -41,25 +41,28 @@ enum {
 	CT_REMOVED     = (1 <<  2),
 	CT_NUM_CURRENT = (1 <<  3),
 	CT_DEP_DONE    = (1 <<  4),
+	CT_AP_UNKNOWN  = (1 <<  5),
 };
 
 constexpr auto DEP_NO_PARENT = std::numeric_limits<uint32_t>::max();
 
 class Cohort {
 public:
-	uint8_t type;
+	uint8_t type = 0;
 	// ToDo: Get rid of global_number in favour of Cohort* relations
-	uint32_t global_number;
-	uint32_t local_number;
-	Tag* wordform;
-	uint32_t dep_self;
-	uint32_t dep_parent;
-	uint32_t is_pleft, is_pright;
-	SingleWindow* parent;
+	uint32_t global_number = 0;
+	uint32_t local_number = 0;
+	Tag* wordform = nullptr;
+	uint32_t dep_self = 0;
+	uint32_t dep_parent = DEP_NO_PARENT;
+	uint32_t is_pleft = 0;
+	uint32_t is_pright = 0;
+	SingleWindow* parent = nullptr;
 	UString text;
 	UString wblank;
-	Cohort *prev, *next;
-	Reading* wread;
+	Cohort* prev = nullptr;
+	Cohort* next = nullptr;
+	Reading* wread = nullptr;
 	ReadingList readings;
 	ReadingList deleted;
 	ReadingList delayed;
