@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -35,10 +35,10 @@ CohortIterator::CohortIterator(Cohort* cohort, const ContextualTest* test, bool
 CohortIterator::~CohortIterator() {
 }
 
-bool CohortIterator::operator==(const CohortIterator& other) {
+bool CohortIterator::operator==(const CohortIterator& other) const {
 	return (m_cohort == other.m_cohort);
 }
-bool CohortIterator::operator!=(const CohortIterator& other) {
+bool CohortIterator::operator!=(const CohortIterator& other) const {
 	return (m_cohort != other.m_cohort);
 }
 
@@ -108,7 +108,7 @@ DepParentIter& DepParentIter::operator++() {
 		return *this;
 	}
 	if (m_cohort->dep_parent != DEP_NO_PARENT) {
-		std::map<uint32_t, Cohort*>::iterator it = m_cohort->parent->parent->cohort_map.find(m_cohort->dep_parent);
+		auto it = m_cohort->parent->parent->cohort_map.find(m_cohort->dep_parent);
 		if (it != m_cohort->parent->parent->cohort_map.end()) {
 			Cohort* cohort = it->second;
 			if (cohort->type & CT_REMOVED) {
@@ -356,10 +356,10 @@ MultiCohortIterator::MultiCohortIterator(Cohort* cohort, const ContextualTest* t
 MultiCohortIterator::~MultiCohortIterator() {
 }
 
-bool MultiCohortIterator::operator==(const MultiCohortIterator& other) {
+bool MultiCohortIterator::operator==(const MultiCohortIterator& other) const {
 	return (m_cohort == other.m_cohort);
 }
-bool MultiCohortIterator::operator!=(const MultiCohortIterator& other) {
+bool MultiCohortIterator::operator!=(const MultiCohortIterator& other) const {
 	return (m_cohort != other.m_cohort);
 }
 
