@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -42,8 +42,8 @@ public:
 
 	virtual ~CohortIterator();
 
-	bool operator==(const CohortIterator& other);
-	bool operator!=(const CohortIterator& other);
+	bool operator==(const CohortIterator& other) const;
+	bool operator!=(const CohortIterator& other) const;
 
 	virtual CohortIterator& operator++();
 
@@ -52,9 +52,9 @@ public:
 	virtual void reset(Cohort* cohort = nullptr, const ContextualTest* test = nullptr, bool span = false);
 
 protected:
-	bool m_span;
-	Cohort* m_cohort;
-	const ContextualTest* m_test;
+	bool m_span = false;
+	Cohort* m_cohort = nullptr;
+	const ContextualTest* m_test = nullptr;
 };
 
 class TopologyLeftIter : public CohortIterator {
@@ -135,8 +135,8 @@ public:
 
 	virtual ~MultiCohortIterator();
 
-	bool operator==(const MultiCohortIterator& other);
-	bool operator!=(const MultiCohortIterator& other);
+	bool operator==(const MultiCohortIterator& other) const;
+	bool operator!=(const MultiCohortIterator& other) const;
 
 	virtual MultiCohortIterator& operator++();
 
