@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -74,7 +74,8 @@ enum : uint64_t {
 	POS_NO_BARRIER     = (1ull << 36), // N
 	POS_WITH           = (1ull << 37), // w
 	POS_LOOK_IGNORED   = (1ull << 38), // I
-	POS_OTHER          = (1ull << 39), // t
+	POS_INACTIVE       = (1ull << 39), // t
+	POS_ACTIVE         = (1ull << 40), // T
 
 	MASK_POS_DEP       = POS_DEP_PARENT | POS_DEP_SIBLING | POS_DEP_CHILD | POS_DEP_GLOB,
 	MASK_POS_DEPREL    = MASK_POS_DEP | POS_RELATION,
@@ -90,25 +91,25 @@ enum GSR_SPECIALS {
 
 class ContextualTest {
 public:
-	bool is_used;
-	int32_t offset;
-	int32_t offset_sub;
-	uint32_t line;
-	uint32_t hash;
-	uint32_t seed;
-	uint64_t pos;
-	uint32_t target;
-	uint32_t relation;
-	uint32_t barrier;
-	uint32_t cbarrier;
-	mutable uint32_t num_fail, num_match;
-	mutable double total_time;
-	ContextualTest* tmpl;
-	ContextualTest* linked;
+	bool is_used = false;
+	int32_t offset = 0;
+	int32_t offset_sub = 0;
+	uint32_t line = 0;
+	uint32_t hash = 0;
+	uint32_t seed = 0;
+	uint64_t pos = 0;
+	uint32_t target = 0;
+	uint32_t relation = 0;
+	uint32_t barrier = 0;
+	uint32_t cbarrier = 0;
+	mutable uint32_t num_fail = 0, num_match = 0;
+	mutable double total_time = 0;
+	ContextualTest* tmpl = nullptr;
+	ContextualTest* linked = nullptr;
 
 	ContextVector ors;
 
-	ContextualTest();
+	ContextualTest() = default;
 
 	bool operator==(const ContextualTest&) const;
 	bool operator!=(const ContextualTest& o) const { return !(*this == o); }
