@@ -195,7 +195,7 @@ void FSTApplicator::runGrammarOnText(std::istream& input, std::ostream& output)
 
 				while (space && *space && (space = u_strchr(space, '+')) != 0) {
 					if (base && base[0]) {
-						int32_t f = u_strcspn(base, sub_delims.c_str());
+						int32_t f = u_strcspn(base, sub_delims.data());
 						UChar* hash = nullptr;
 						if (f && base + f < space) {
 							hash = const_cast<UChar*>(base) + f;
@@ -214,7 +214,7 @@ void FSTApplicator::runGrammarOnText(std::istream& input, std::ostream& output)
 							tag += '"';
 							tag += base;
 							tag += '"';
-							base = tag.c_str();
+							base = tag.data();
 						}
 						if (base[0] == 0) {
 							base = notag;
@@ -246,7 +246,7 @@ void FSTApplicator::runGrammarOnText(std::istream& input, std::ostream& output)
 						tag += '"';
 						tag += base;
 						tag += '"';
-						base = tag.c_str();
+						base = tag.data();
 					}
 					Tag* tag = addTag(base);
 					if (tag->type & T_MAPPING || tag->tag[0] == grammar->mapping_prefix) {
@@ -330,7 +330,7 @@ void FSTApplicator::runGrammarOnText(std::istream& input, std::ostream& output)
 			}
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (!dep_delimit && grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.data(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -415,4 +415,115 @@ void FSTApplicator::runGrammarOnText(std::istream& input, std::ostream& output)
 
 	u_fflush(output);
 }
+void FSTApplicator::printReading(const Reading* reading, std::ostream& output) {
+	if (reading->noprint) {
+		return;
+	}
+	if (reading->deleted) {
+		return;
+	}
+
+	if (reading->next) {
+		printReading(reading->next, output);
+		u_fprintf(output, "%S", sub_delims.data());
+	}
+
+	if (reading->baseform) {
+		auto& tag = grammar->single_tags.find(reading->baseform)->second->tag;
+		u_fprintf(output, "%.*S", tag.size() - 2, tag.data() + 1);
+	}
+
+	uint32SortedVector unique;
+	for (auto tter : reading->tags_list) {
+		if ((!show_end_tags && tter == endtag) || tter == begintag) {
+			continue;
+		}
+		if (tter == reading->baseform || tter == reading->parent->wordform->hash) {
+			continue;
+		}
+		if (unique_tags) {
+			if (unique.find(tter) != unique.end()) {
+				continue;
+			}
+			unique.insert(tter);
+		}
+		const Tag* tag = grammar->single_tags[tter];
+		if (tag->type & T_DEPENDENCY && has_dep && !dep_original) {
+			continue;
+		}
+		if (tag->type & T_RELATION && has_relations) {
+			continue;
+		}
+		u_fprintf(output, "+%S", tag->tag.data());
+	}
+}
+
+void FSTApplicator::printCohort(Cohort* cohort, std::ostream& output) {
+	if (cohort->local_number == 0) {
+		goto removed;
+	}
+	if (cohort->type & CT_REMOVED) {
+		goto removed;
+	}
+
+	if (cohort->wread && !did_warn_statictags) {
+		u_fprintf(ux_stderr, "Warning: FST CG format cannot output static tags! You are losing information!\n");
+		u_fflush(ux_stderr);
+		did_warn_statictags = true;
+	}
+
+	cohort->unignoreAll();
+
+	if (!split_mappings) {
+		mergeMappings(*cohort);
+	}
+
+	{
+		auto& wform = cohort->wordform->tag;
+		if (cohort->readings.empty() || (cohort->readings.size() == 1 && cohort->readings[0]->noprint)) {
+			u_fprintf(output, "%.*S\t+?\n", wform.size() - 4, wform.data() + 2);
+		}
+		else {
+			for (auto rter : cohort->readings) {
+				u_fprintf(output, "%.*S\t", wform.size() - 4, wform.data() + 2);
+				printReading(rter, output);
+				u_fputc('\n', output);
+			}
+		}
+		u_fputc('\n', output);
+	}
+
+removed:
+	if (!cohort->text.empty() && cohort->text.find_first_not_of(ws) != UString::npos) {
+		u_fprintf(output, "%S", cohort->text.data());
+		if (!ISNL(cohort->text.back())) {
+			u_fputc('\n', output);
+		}
+	}
+}
+
+void FSTApplicator::printSingleWindow(SingleWindow* window, std::ostream& output) {
+	if (!window->text.empty()) {
+		u_fprintf(output, "%S", window->text.data());
+		if (!ISNL(window->text.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
+	uint32_t cs = UI32(window->cohorts.size());
+	for (uint32_t c = 0; c < cs; c++) {
+		Cohort* cohort = window->cohorts[c];
+		printCohort(cohort, output);
+	}
+
+	if (!window->text_post.empty()) {
+		u_fprintf(output, "%S", window->text_post.data());
+		if (!ISNL(window->text_post.back())) {
+			u_fputc('\n', output);
+		}
+	}
+
+	u_fputc('\n', output);
+	u_fflush(output);
+}
 }
