@@ -114,6 +114,8 @@ public:
 
 	void addSet(Set*& to);
 	Set* getSet(uint32_t which) const;
+	void appendToSet(Set*& to);
+	Set* undefSet(const UString& name);
 	Set* allocateSet();
 	void destroySet(Set* set);
 	void addSetToList(Set* s);
@@ -125,10 +127,16 @@ public:
 
 	void addAnchor(const UChar* to, uint32_t at, bool primary = false);
 	void addAnchor(const UString& to, uint32_t at, bool primary = false);
+	void addAnchor(const UStringView& to, uint32_t at, bool primary = false) {
+		return addAnchor(to.data(), at, primary);
+	}
 
 	Tag* allocateTag();
 	Tag* allocateTag(const UChar* tag);
 	Tag* allocateTag(const UString& tag);
+	Tag* allocateTag(const UStringView& tag) {
+		return allocateTag(tag.data());
+	}
 	Tag* addTag(Tag* tag);
 	void destroyTag(Tag* tag);
 	void addTagToSet(Tag* rtag, Set* set);
@@ -169,7 +177,7 @@ inline void _trie_unserialize(trie_t& trie, Stream& input, Grammar& grammar, uin
 		u32tmp = ntoh32(u32tmp);
 		if (u32tmp) {
 			if (!node.trie) {
-				node.trie = new trie_t;
+				node.trie.reset(new trie_t);
 			}
 			trie_unserialize(*node.trie, input, grammar, u32tmp);
 		}
