@@ -58,6 +58,30 @@ inline bool TagSet_SubsetOf_TSet(const TagSortedVector& a, const T& b) {
 	return true;
 }
 
+template<typename RXGS, typename Tag>
+inline void captureRegex(int32_t gc, RXGS& regexgrps, Tag& tag) {
+	constexpr auto BUFSIZE = 1024;
+	UErrorCode status = U_ZERO_ERROR;
+	UChar _tmp[BUFSIZE];
+	UString _stmp;
+	UChar* tmp = _tmp;
+	for (int i = 1; i <= gc; ++i) {
+		tmp[0] = 0;
+		int32_t len = uregex_group(tag.regexp, i, tmp, BUFSIZE, &status);
+		if (len >= BUFSIZE) {
+			status = U_ZERO_ERROR;
+			_stmp.resize(len + 1);
+			tmp = &_stmp[0];
+			uregex_group(tag.regexp, i, tmp, len + 1, &status);
+		}
+		regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
+		UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
+		ucstr.remove();
+		ucstr.append(tmp, len);
+		++regexgrps.first;
+	}
+}
+
 /**
 * Tests whether a given input tag matches a given tag's stored regular expression.
 *
@@ -77,9 +101,9 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 	}
 	else {
 		const Tag& itag = *(grammar->single_tags.find(test)->second);
-		uregex_setText(tag.regexp, itag.tag.c_str(), SI32(itag.tag.size()), &status);
+		uregex_setText(tag.regexp, itag.tag.data(), SI32(itag.tag.size()), &status);
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_setText(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_setText(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 			CG3Quit(1);
 		}
 		status = U_ZERO_ERROR;
@@ -87,21 +111,12 @@ uint32_t GrammarApplicator::doesTagMatchRegexp(uint32_t test, const Tag& tag, bo
 			match = itag.hash;
 		}
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_find(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_find(MatchTag) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 			CG3Quit(1);
 		}
 		if (match) {
 			if (gc > 0 && regexgrps.second != 0) {
-				UChar tmp[1024];
-				for (int i = 1; i <= gc; ++i) {
-					tmp[0] = 0;
-					int32_t len = uregex_group(tag.regexp, i, tmp, 1024, &status);
-					regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
-					UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
-					ucstr.remove();
-					ucstr.append(tmp, len);
-					++regexgrps.first;
-				}
+				captureRegex(gc, regexgrps, tag);
 			}
 			else {
 				index_regexp_yes.insert(ih);
@@ -151,9 +166,9 @@ uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Ta
 		match = reading.tags_string_hash;
 	}
 	else {
-		uregex_setText(tag.regexp, reading.tags_string.c_str(), SI32(reading.tags_string.size()), &status);
+		uregex_setText(tag.regexp, reading.tags_string.data(), SI32(reading.tags_string.size()), &status);
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_setText(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_setText(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 			CG3Quit(1);
 		}
 		status = U_ZERO_ERROR;
@@ -161,22 +176,13 @@ uint32_t GrammarApplicator::doesRegexpMatchLine(const Reading& reading, const Ta
 			match = reading.tags_string_hash;
 		}
 		if (status != U_ZERO_ERROR) {
-			u_fprintf(ux_stderr, "Error: uregex_find(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+			u_fprintf(ux_stderr, "Error: uregex_find(MatchLine) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 			CG3Quit(1);
 		}
 		if (match) {
 			// ToDo: Allow regex captures from dependency target contexts without any captures in normal target contexts
 			if (gc > 0 && regexgrps.second != 0) {
-				UChar tmp[1024];
-				for (int i = 1; i <= gc; ++i) {
-					tmp[0] = 0;
-					int32_t len = uregex_group(tag.regexp, i, tmp, 1024, &status);
-					regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
-					UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
-					ucstr.remove();
-					ucstr.append(tmp, len);
-					++regexgrps.first;
-				}
+				captureRegex(gc, regexgrps, tag);
 			}
 			else {
 				index_regexp_yes.insert(ih);
@@ -255,9 +261,9 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 	else if (tag.type & T_META) {
 		if (tag.regexp && !reading.parent->text.empty()) {
 			UErrorCode status = U_ZERO_ERROR;
-			uregex_setText(tag.regexp, reading.parent->text.c_str(), SI32(reading.parent->text.size()), &status);
+			uregex_setText(tag.regexp, reading.parent->text.data(), SI32(reading.parent->text.size()), &status);
 			if (status != U_ZERO_ERROR) {
-				u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+				u_fprintf(ux_stderr, "Error: uregex_setText(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 				CG3Quit(1);
 			}
 			status = U_ZERO_ERROR;
@@ -265,22 +271,13 @@ uint32_t GrammarApplicator::doesTagMatchReading(const Reading& reading, const Ta
 				match = tag.hash;
 			}
 			if (status != U_ZERO_ERROR) {
-				u_fprintf(ux_stderr, "Error: uregex_find(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.c_str(), numLines);
+				u_fprintf(ux_stderr, "Error: uregex_find(MatchSet) returned %s for tag %S before input line %u - cannot continue!\n", u_errorName(status), tag.tag.data(), numLines);
 				CG3Quit(1);
 			}
 			if (match) {
 				int32_t gc = uregex_groupCount(tag.regexp, &status);
 				if (gc > 0 && regexgrps.second != 0) {
-					UChar tmp[1024];
-					for (int i = 1; i <= gc; ++i) {
-						tmp[0] = 0;
-						int32_t len = uregex_group(tag.regexp, i, tmp, 1024, &status);
-						regexgrps.second->resize(std::max(static_cast<size_t>(regexgrps.first) + 1, regexgrps.second->size()));
-						UnicodeString& ucstr = (*regexgrps.second)[regexgrps.first];
-						ucstr.remove();
-						ucstr.append(tmp, len);
-						++regexgrps.first;
-					}
+					captureRegex(gc, regexgrps, tag);
 				}
 			}
 		}
@@ -724,15 +721,8 @@ bool GrammarApplicator::doesSetMatchReading(const Reading& reading, const uint32
 						}
 					}
 					break;
-				case S_NOT:
-					if (!match) {
-						if (!doesSetMatchReading(reading, theset.sets[i + 1], bypass_index, ((theset.type & ST_TAG_UNIFY) != 0) | unif_mode)) {
-							match = true;
-						}
-					}
-					break;
 				default:
-					break;
+					throw std::runtime_error("Set operator not implemented!");
 				}
 				++i;
 			}
@@ -907,8 +897,6 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 		lists[3] = &cohort.ignored;
 	}
 
-	bool non_match_was_sub = false;
-
 	for (auto list : lists) {
 		if (list == nullptr) {
 			continue;
@@ -918,11 +906,13 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 				// ToDo: Barriers need some way to escape sub-readings
 				reading = get_sub_reading(reading, context->test->offset_sub);
 				if (!reading) {
-					non_match_was_sub = true;
 					continue;
 				}
 			}
-			if (reading->active && context && (context->options & POS_OTHER)) {
+			if (!reading->active && context && (context->options & POS_ACTIVE)) {
+				continue;
+			}
+			if (reading->active && context && (context->options & POS_INACTIVE)) {
 				continue;
 			}
 			if (doesSetMatchCohort_helper(cohort, *reading, *theset, context)) {
@@ -940,7 +930,8 @@ bool GrammarApplicator::doesSetMatchCohortNormal(Cohort& cohort, const uint32_t
 
 	if (context && !context->matched_target) {
 		if (!grammar->sets_any || set >= grammar->sets_any->size() || !grammar->sets_any->test(set)) {
-			if (!non_match_was_sub && set < cohort.possible_sets.size()) {
+			bool was_sub = (context->test && (context->test->offset_sub != 0));
+			if (!was_sub && set < cohort.possible_sets.size()) {
 				cohort.possible_sets.reset(set);
 			}
 		}
@@ -981,7 +972,10 @@ bool GrammarApplicator::doesSetMatchCohortCareful(Cohort& cohort, const uint32_t
 					continue;
 				}
 			}
-			if (reading->active && context && (context->options & POS_OTHER)) {
+			if (!reading->active && context && (context->options & POS_ACTIVE)) {
+				continue;
+			}
+			if (reading->active && context && (context->options & POS_INACTIVE)) {
 				continue;
 			}
 			retval = doesSetMatchCohort_helper(cohort, *reading, *theset, context);
