@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -30,7 +30,7 @@
 namespace CG3 {
 
 Tag* GrammarApplicator::makeBaseFromWord(uint32_t tag) {
-	return makeBaseFromWord(single_tags.find(tag)->second);
+	return makeBaseFromWord(grammar->single_tags.find(tag)->second);
 }
 
 Tag* GrammarApplicator::makeBaseFromWord(Tag* tag) {
@@ -42,7 +42,7 @@ Tag* GrammarApplicator::makeBaseFromWord(Tag* tag) {
 	n.clear();
 	n.resize(len - 2);
 	n[0] = n[len - 3] = '"';
-	u_strncpy(&n[1], tag->tag.c_str() + 2, static_cast<int32_t>(len - 4));
+	u_strncpy(&n[1], tag->tag.data() + 2, SI32(len - 4));
 	Tag* nt = addTag(n);
 	return nt;
 }
@@ -63,7 +63,7 @@ bool GrammarApplicator::isChildOf(const Cohort* child, const Cohort* parent) {
 				retval = false;
 				break;
 			}
-			std::map<uint32_t, Cohort*>::iterator it = gWindow->cohort_map.find(inner->dep_parent);
+			auto it = gWindow->cohort_map.find(inner->dep_parent);
 			if (it != gWindow->cohort_map.end()) {
 				inner = it->second;
 			}
@@ -109,7 +109,7 @@ bool GrammarApplicator::wouldParentChildLoop(const Cohort* parent, const Cohort*
 				retval = false;
 				break;
 			}
-			std::map<uint32_t, Cohort*>::iterator it = gWindow->cohort_map.find(inner->dep_parent);
+			auto it = gWindow->cohort_map.find(inner->dep_parent);
 			if (it != gWindow->cohort_map.end()) {
 				inner = it->second;
 			}
@@ -138,7 +138,7 @@ bool GrammarApplicator::wouldParentChildCross(const Cohort* parent, const Cohort
 	uint32_t mx = std::max(parent->global_number, child->global_number);
 
 	for (uint32_t i = mn + 1; i < mx; ++i) {
-		std::map<uint32_t, Cohort*>::iterator it = gWindow->cohort_map.find(parent->dep_parent);
+		auto it = gWindow->cohort_map.find(parent->dep_parent);
 		if (it != gWindow->cohort_map.end() && it->second->dep_parent != DEP_NO_PARENT) {
 			if (it->second->dep_parent < mn || it->second->dep_parent > mx) {
 				return true;
@@ -176,7 +176,7 @@ bool GrammarApplicator::attachParentChild(Cohort& parent, Cohort& child, bool al
 	if (child.dep_parent == DEP_NO_PARENT) {
 		child.dep_parent = child.dep_self;
 	}
-	std::map<uint32_t, Cohort*>::iterator it = gWindow->cohort_map.find(child.dep_parent);
+	auto it = gWindow->cohort_map.find(child.dep_parent);
 	if (it != gWindow->cohort_map.end()) {
 		it->second->remChild(child.dep_self);
 	}
@@ -307,11 +307,11 @@ void GrammarApplicator::reflowRelationWindow(uint32_t max) {
 			break;
 		}
 
-		for (RelationCtn::iterator rel = cohort->relations_input.begin(); rel != cohort->relations_input.end();) {
+		for (auto rel = cohort->relations_input.begin(); rel != cohort->relations_input.end();) {
 			auto newrel = ss_u32sv.get();
 
 			for (auto target : rel->second) {
-				uint32FlatHashMap::iterator it = gWindow->relation_map.find(target);
+				auto it = gWindow->relation_map.find(target);
 				if (it != gWindow->relation_map.end()) {
 					cohort->relations[rel->first].insert(it->second);
 				}
@@ -322,7 +322,7 @@ void GrammarApplicator::reflowRelationWindow(uint32_t max) {
 
 			// Defer missing relations for later
 			if (newrel->empty()) {
-				cohort->relations_input.erase(rel++);
+				rel = cohort->relations_input.erase(rel);
 			}
 			else {
 				rel->second = newrel;
@@ -358,12 +358,14 @@ void GrammarApplicator::reflowReading(Reading& reading) {
 Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 	static thread_local UnicodeString tmp;
 	tmp.remove();
-	tmp.append(tag->tag.c_str(), static_cast<int32_t>(tag->tag.size()));
+	tmp.append(tag->tag.data(), SI32(tag->tag.size()));
 	bool did_something = false;
 
 	// Convert %[UuLl] markers to control codes to avoid having combined %$1 accidentally match %L
+	constexpr UStringView raw[] = { STR_VSu_raw, STR_VSU_raw, STR_VSl_raw, STR_VSL_raw };
+	constexpr UStringView x01[] = { STR_VSu, STR_VSU, STR_VSl, STR_VSL };
 	for (size_t i = 0; i < 4; ++i) {
-		tmp.findAndReplace(stringbits[S_VSu_raw + i].c_str(), stringbits[S_VSu + i].c_str());
+		findAndReplace(tmp, raw[i].data(), x01[i].data());
 	}
 
 	// Replace unified sets with their matching tags
@@ -380,14 +382,15 @@ Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 					rpl += '_';
 				}
 			}
-			tmp.findAndReplace((*tag->vs_names)[i].c_str(), rpl.c_str());
+			findAndReplace(tmp, (*tag->vs_names)[i].data(), rpl.data());
 			did_something = true;
 		}
 	}
 
 	// Replace $1-$9 with their respective match groups
+	constexpr UStringView grp[] = { STR_VS1, STR_VS2, STR_VS3, STR_VS4, STR_VS5, STR_VS6, STR_VS7, STR_VS8, STR_VS9 };
 	for (size_t i = 0; i < regexgrps.first && i < 9; ++i) {
-		tmp.findAndReplace(stringbits[S_VS1 + i].c_str(), (*regexgrps.second)[i]);
+		findAndReplace(tmp, grp[i].data(), USV((*regexgrps.second)[i]));
 		did_something = true;
 	}
 
@@ -396,19 +399,19 @@ Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 	do {
 		found = false;
 		int32_t pos = -1, mpos = -1;
-		if ((pos = tmp.lastIndexOf(stringbits[S_VSu].c_str(), stringbits[S_VSu].size(), 0)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSu.data(), STR_VSu.size(), 0)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(stringbits[S_VSU].c_str(), stringbits[S_VSU].size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSU.data(), STR_VSU.size(), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(stringbits[S_VSl].c_str(), stringbits[S_VSl].size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSl.data(), STR_VSl.size(), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
-		if ((pos = tmp.lastIndexOf(stringbits[S_VSL].c_str(), stringbits[S_VSL].size(), mpos)) != -1) {
+		if ((pos = tmp.lastIndexOf(STR_VSL.data(), STR_VSL.size(), mpos)) != -1) {
 			found = true;
 			mpos = std::max(mpos, pos);
 		}
@@ -450,14 +453,14 @@ Tag* GrammarApplicator::generateVarstringTag(const Tag* tag) {
 
 	const UChar* nt = tmp.getTerminatedBuffer();
 	if (!did_something && nt == tag->tag) {
-		u_fprintf(ux_stderr, "Warning: Unable to generate from tag '%S'! Possibly missing KEEPORDER and/or capturing regex from grammar on line %u before input line %u.\n", tag->tag.c_str(), grammar->lines, numLines);
+		u_fprintf(ux_stderr, "Warning: Unable to generate from tag '%S'! Possibly missing KEEPORDER and/or capturing regex from grammar on line %u before input line %u.\n", tag->tag.data(), grammar->lines, numLines);
 		u_fflush(ux_stderr);
 	}
 	return addTag(nt, true);
 }
 
 uint32_t GrammarApplicator::addTagToReading(Reading& reading, uint32_t utag, bool rehash) {
-	Tag* tag = single_tags.find(utag)->second;
+	Tag* tag = grammar->single_tags.find(utag)->second;
 	return addTagToReading(reading, tag, rehash);
 }
 
@@ -466,7 +469,7 @@ uint32_t GrammarApplicator::addTagToReading(Reading& reading, Tag* tag, bool reh
 		tag = generateVarstringTag(tag);
 	}
 
-	Grammar::sets_by_tag_t::const_iterator it = grammar->sets_by_tag.find(tag->hash);
+	auto it = grammar->sets_by_tag.find(tag->hash);
 	if (it != grammar->sets_by_tag.end()) {
 		reading.parent->possible_sets.resize(std::max(reading.parent->possible_sets.size(), it->second.size()));
 		reading.parent->possible_sets |= it->second;
@@ -599,7 +602,7 @@ bool GrammarApplicator::unmapReading(Reading& reading, const uint32_t rule) {
 }
 
 void GrammarApplicator::splitMappings(TagList& mappings, Cohort& cohort, Reading& reading, bool mapped) {
-	for (TagList::iterator it = mappings.begin(); it != mappings.end();) {
+	for (auto it = mappings.begin(); it != mappings.end();) {
 		Tag*& tag = *it;
 		while (tag->type & T_VARSTRING) {
 			tag = generateVarstringTag(tag);
@@ -635,10 +638,10 @@ void GrammarApplicator::splitMappings(TagList& mappings, Cohort& cohort, Reading
 		}
 		Reading* nr = alloc_reading(reading);
 		nr->mapped = mapped;
-		nr->number = static_cast<uint32_t>(reading.number - i--);
+		nr->number = UI32(reading.number - i--);
 		uint32_t mp = addTagToReading(*nr, ttag);
 		if (mp != ttag->hash) {
-			nr->mapping = single_tags.find(mp)->second;
+			nr->mapping = grammar->single_tags.find(mp)->second;
 		}
 		else {
 			nr->mapping = ttag;
@@ -650,7 +653,7 @@ void GrammarApplicator::splitMappings(TagList& mappings, Cohort& cohort, Reading
 	reading.mapped = mapped;
 	uint32_t mp = addTagToReading(reading, tag);
 	if (mp != tag->hash) {
-		reading.mapping = single_tags.find(mp)->second;
+		reading.mapping = grammar->single_tags.find(mp)->second;
 	}
 	else {
 		reading.mapping = tag;
@@ -797,6 +800,8 @@ Cohort* GrammarApplicator::delimitAt(SingleWindow& current, Cohort* cohort) {
 
 	assert(nwin != 0);
 
+	std::swap(current.flush_after, nwin->flush_after);
+	std::swap(current.text_post, nwin->text_post);
 	nwin->has_enclosures = current.has_enclosures;
 
 	Cohort* cCohort = alloc_cohort(nwin);
@@ -818,7 +823,7 @@ Cohort* GrammarApplicator::delimitAt(SingleWindow& current, Cohort* cohort) {
 		current.cohorts[nc]->parent = nwin;
 		nwin->appendCohort(current.cohorts[nc]);
 	}
-	c = static_cast<uint32_t>(current.cohorts.size() - c);
+	c = UI32(current.cohorts.size() - c);
 	for (nc = 0; nc < c - 1; nc++) {
 		current.cohorts.pop_back();
 	}
@@ -837,7 +842,7 @@ void GrammarApplicator::reflowTextuals_Reading(Reading& r) {
 		reflowTextuals_Reading(*r.next);
 	}
 	for (auto it : r.tags) {
-		Tag* tag = single_tags.find(it)->second;
+		Tag* tag = grammar->single_tags.find(it)->second;
 		if (tag->type & T_TEXTUAL) {
 			r.tags_textual.insert(it);
 			r.tags_textual_bloom.insert(it);
