@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -69,7 +69,7 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 		*deep = cohort;
 	}
 
-	dSMC_Context context = { test, deep, origin, test->pos, false, false, false };
+	dSMC_Context context = { test, deep, origin, test->pos };
 
 	if (test->pos & POS_CAREFUL) {
 		*retval = doesSetMatchCohortCareful(*cohort, test->target, &context);
@@ -84,9 +84,7 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 	}
 
 	if (origin && (test->offset != 0 || (test->pos & (POS_SCANALL | POS_SCANFIRST))) && origin == cohort && origin->local_number != 0) {
-		if (!(test->pos & POS_NOT)) {
-			*retval = false;
-		}
+		cohort = nullptr;
 		rvs |= TRV_BREAK;
 	}
 	if (context.matched_target && (test->pos & POS_SCANFIRST)) {
@@ -102,7 +100,7 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 	context.deep = nullptr;
 	context.origin = nullptr;
 	context.did_test = true;
-	if (test->barrier) {
+	if (test->barrier && cohort) {
 		dSMC_Context context = { nullptr, nullptr, nullptr, test->pos & ~POS_CAREFUL, false, false, false, true };
 		bool barrier = doesSetMatchCohortNormal(*cohort, test->barrier, &context);
 		if (barrier) {
@@ -111,7 +109,7 @@ Cohort* GrammarApplicator::runSingleTest(Cohort* cohort, const ContextualTest* t
 			rvs &= ~TRV_BREAK_DEFAULT;
 		}
 	}
-	if (test->cbarrier) {
+	if (test->cbarrier && cohort) {
 		dSMC_Context context = { nullptr, nullptr, nullptr, test->pos | POS_CAREFUL, false, false, false, true };
 		bool cbarrier = doesSetMatchCohortCareful(*cohort, test->cbarrier, &context);
 		if (cbarrier) {
@@ -144,7 +142,7 @@ Cohort* GrammarApplicator::runSingleTest(SingleWindow* sWindow, size_t i, const
 
 Cohort* getCohortInWindow(SingleWindow*& sWindow, size_t position, const ContextualTest* test, int32_t& pos) {
 	Cohort* cohort = nullptr;
-	pos = static_cast<int32_t>(position) + test->offset;
+	pos = SI32(position) + test->offset;
 	// ToDo: (NOT*) and (*C) tests can be cached
 	if ((test->pos & POS_ABSOLUTE) && (test->pos & (POS_SPAN_LEFT|POS_SPAN_RIGHT))) {
 		if (sWindow->previous && (test->pos & POS_SPAN_LEFT)) {
@@ -159,14 +157,14 @@ Cohort* getCohortInWindow(SingleWindow*& sWindow, size_t position, const Context
 	}
 	if (test->pos & POS_ABSOLUTE) {
 		if (test->offset < 0) {
-			pos = static_cast<int32_t>(sWindow->cohorts.size()) + test->offset;
+			pos = SI32(sWindow->cohorts.size()) + test->offset;
 		}
 		else {
 			pos = test->offset;
 		}
 	}
 	if (pos >= 0) {
-		if (pos >= static_cast<int32_t>(sWindow->cohorts.size()) && (test->pos & (POS_SPAN_RIGHT | POS_SPAN_BOTH)) && sWindow->next) {
+		if (pos >= SI32(sWindow->cohorts.size()) && (test->pos & (POS_SPAN_RIGHT | POS_SPAN_BOTH)) && sWindow->next) {
 			sWindow = sWindow->next;
 			pos = 0;
 		}
@@ -174,10 +172,10 @@ Cohort* getCohortInWindow(SingleWindow*& sWindow, size_t position, const Context
 	else {
 		if ((test->pos & (POS_SPAN_LEFT | POS_SPAN_BOTH)) && sWindow->previous) {
 			sWindow = sWindow->previous;
-			pos = static_cast<int32_t>(sWindow->cohorts.size()) - 1;
+			pos = SI32(sWindow->cohorts.size()) - 1;
 		}
 	}
-	if (pos >= 0 && pos < static_cast<int32_t>(sWindow->cohorts.size())) {
+	if (pos >= 0 && pos < SI32(sWindow->cohorts.size())) {
 		cohort = sWindow->cohorts[pos];
 	}
 	return cohort;
@@ -207,11 +205,11 @@ bool GrammarApplicator::posOutputHelper(const SingleWindow* sWindow, size_t posi
 	}
 	else {
 		// ...otherwise, positive offsets need to match the leftmost of entry/exit
-		if (test->offset > 0 && static_cast<int32_t>(cs[0]->local_number) - static_cast<int32_t>(position) == test->offset) {
+		if (test->offset > 0 && SI32(cs[0]->local_number) - SI32(position) == test->offset) {
 			good = true;
 		}
 		// ...and, negative offsets need to match the rightmost of entry/exit
-		else if (test->offset < 0 && static_cast<int32_t>(cs[3]->local_number) - static_cast<int32_t>(position) == test->offset) {
+		else if (test->offset < 0 && SI32(cs[3]->local_number) - SI32(position) == test->offset) {
 			good = true;
 		}
 	}
@@ -473,7 +471,7 @@ Cohort* GrammarApplicator::runContextualTest(SingleWindow* sWindow, size_t posit
 						if ((test->pos & (POS_SPAN_BOTH | POS_SPAN_LEFT) || always_span)) {
 							left = left->previous;
 							if (left) {
-								lpos = static_cast<int32_t>(i + left->cohorts.size());
+								lpos = SI32(i + left->cohorts.size());
 							}
 						}
 						else {
@@ -522,7 +520,7 @@ Cohort* GrammarApplicator::runContextualTest(SingleWindow* sWindow, size_t posit
 			size_t seen = 0;
 			if ((test->pos & POS_SELF) && (!(test->pos & MASK_POS_LORR) || ((test->pos & POS_DEP_PARENT) && !(test->pos & POS_DEP_GLOB)))) {
 				++seen;
-				assert(pos >= 0 && pos < static_cast<int32_t>(sWindow->cohorts.size()) && "Somehow, the input position wasn't inside the current window.");
+				assert(pos >= 0 && pos < SI32(sWindow->cohorts.size()) && "Somehow, the input position wasn't inside the current window.");
 				Cohort* self = sWindow->cohorts[position];
 				nc = runSingleTest(self, test, rvs, &retval, deep, origin);
 				if (!retval && (rvs & TRV_BREAK_DEFAULT)) {
@@ -644,7 +642,7 @@ Cohort* GrammarApplicator::runDependencyTest(SingleWindow* sWindow, Cohort* curr
 			deps = &(current->parent->cohorts[0]->dep_children);
 		}
 		else {
-			std::map<uint32_t, Cohort*>::iterator it = current->parent->parent->cohort_map.find(current->dep_parent);
+			auto it = current->parent->parent->cohort_map.find(current->dep_parent);
 			if (it != current->parent->parent->cohort_map.end() && it->second && !it->second->dep_children.empty()) {
 				deps = &(it->second->dep_children);
 			}
@@ -661,7 +659,7 @@ Cohort* GrammarApplicator::runDependencyTest(SingleWindow* sWindow, Cohort* curr
 	// ToDo: This whole function could resolve cohorts earlier and skip doing it twice
 	if (test->pos & MASK_POS_LORR) {
 		// I think this way around makes most sense? Loop over the container that's slower to look up in. But tests will show.
-		for (auto iter : sWindow->parent->cohort_map) {
+		for (const auto& iter : sWindow->parent->cohort_map) {
 			if (deps->count(iter.second->global_number)) {
 				if (test->pos & POS_LEFT) {
 					if (less_Cohort(iter.second, current)) {
@@ -782,27 +780,41 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 
 	// Recursion may happen, so can't be static
 	CohortSet rels;
+	uint8_t regexgrpz = regexgrps.first;
 
-	auto rtag = single_tags[test->relation];
+	auto rtag = grammar->single_tags[test->relation];
 	while (rtag->type & T_VARSTRING) {
 		rtag = generateVarstringTag(rtag);
 	}
 
 	if (rtag->hash == grammar->tag_any) {
-		for (auto riter : current->relations) {
+		for (const auto& riter : current->relations) {
 			for (auto citer : riter.second) {
-				std::map<uint32_t, Cohort*>::iterator it = sWindow->parent->cohort_map.find(citer);
+				auto it = sWindow->parent->cohort_map.find(citer);
 				if (it != sWindow->parent->cohort_map.end()) {
 					rels.insert(it->second);
 				}
 			}
 		}
 	}
+	else if (rtag->type & T_REGEXP) {
+		UErrorCode status = U_ZERO_ERROR;
+		auto caps = uregex_groupCount(rtag->regexp, &status);
+		for (const auto& riter : current->relations) {
+			for (auto citer : riter.second) {
+				auto it = sWindow->parent->cohort_map.find(citer);
+				if (it != sWindow->parent->cohort_map.end() && doesTagMatchRegexp(riter.first, *rtag, caps != 0)) {
+					rels.insert(it->second);
+					regexgrps.first = std::min(regexgrps.first, UI8(regexgrpz + caps));
+				}
+			}
+		}
+	}
 	else {
-		RelationCtn::const_iterator riter = current->relations.find(rtag->hash);
+		auto riter = current->relations.find(rtag->hash);
 		if (riter != current->relations.end()) {
 			for (auto citer : riter->second) {
-				std::map<uint32_t, Cohort*>::iterator it = sWindow->parent->cohort_map.find(citer);
+				auto it = sWindow->parent->cohort_map.find(citer);
 				if (it != sWindow->parent->cohort_map.end()) {
 					rels.insert(it->second);
 				}
@@ -855,6 +867,9 @@ Cohort* GrammarApplicator::runRelationTest(SingleWindow* sWindow, Cohort* curren
 		}
 	}
 
+	if (!rv) {
+		regexgrps.first = regexgrpz;
+	}
 	return rv;
 }
 }
