@@ -593,7 +593,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				target = nullptr;
 				mark = cohort;
 				uint8_t orz = regexgrps.first;
-				reading->active = true;
+				for (auto r = cohort->readings[i]; r; r = r->next) {
+					r->active = true;
+				}
 				// Actually check if the reading is a valid target. First check if rule target matches...
 				if (rule.target && doesSetMatchReading(*reading, rule.target, (set.type & (ST_CHILD_UNIFY | ST_SPECIAL)) != 0)) {
 					bool regex_prop = true;
@@ -682,7 +684,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 					++rule.num_fail;
 				}
 				readings_plain.insert(std::make_pair(reading->hash_plain, reading));
-				reading->active = false;
+				for (auto r = cohort->readings[i]; r; r = r->next) {
+					r->active = false;
+				}
 
 				if (reading != cohort->readings[i]) {
 					cohort->readings[i]->matched_target = reading->matched_target;
@@ -893,7 +897,12 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 				// ToDo: Named relations must be erased
 				while (!cohort->dep_children.empty()) {
 					uint32_t ch = cohort->dep_children.back();
-					attachParentChild(*current.parent->cohort_map[cohort->dep_parent], *current.parent->cohort_map[ch], true, true);
+					if (cohort->dep_parent == DEP_NO_PARENT) {
+						attachParentChild(*current.parent->cohort_map[0], *current.parent->cohort_map[ch], true, true);
+					}
+					else {
+						attachParentChild(*current.parent->cohort_map[cohort->dep_parent], *current.parent->cohort_map[ch], true, true);
+					}
 					cohort->dep_children.erase(ch);
 				}
 				cohort->type |= CT_REMOVED;
@@ -917,7 +926,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 			auto make_relation_rtag = [&](Tag* tag, uint32_t id) {
 				UChar tmp[256] = { 0 };
-				u_sprintf(tmp, "R:%S:%u", tag->tag.c_str(), id);
+				u_sprintf(tmp, "R:%S:%u", tag->tag.data(), id);
 				auto nt = addTag(tmp);
 				return nt;
 			};
@@ -1078,7 +1087,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						}
 						auto it = grammar->anchors.find(to->hash);
 						if (it == grammar->anchors.end()) {
-							u_fprintf(ux_stderr, "Warning: JUMP on line %u could not find anchor '%S'.\n", rule.line, to->tag.c_str());
+							u_fprintf(ux_stderr, "Warning: JUMP on line %u could not find anchor '%S'.\n", rule.line, to->tag.data());
 						}
 						else {
 							iter_rules = intersects.lower_bound(it->second);
@@ -1101,7 +1110,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 							if (rule.flags & RF_OUTPUT) {
 								current.variables_output.insert(tag->hash);
 							}
-							//u_fprintf(ux_stderr, "Info: RemVariable fired for %S.\n", tag->tag.c_str());
+							//u_fprintf(ux_stderr, "Info: RemVariable fired for %S.\n", tag->tag.data());
 						}
 						break;
 					}
@@ -1113,7 +1122,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						if (rule.flags & RF_OUTPUT) {
 							current.variables_output.insert(names.front()->hash);
 						}
-						//u_fprintf(ux_stderr, "Info: SetVariable fired for %S.\n", names.front()->tag.c_str());
+						//u_fprintf(ux_stderr, "Info: SetVariable fired for %S.\n", names.front()->tag.data());
 						break;
 					}
 					else if (type == K_DELIMIT) {
@@ -1131,7 +1140,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 						if (ei == externals.end()) {
 							Tag* ext = grammar->single_tags.find(rule.varname)->second;
 							UErrorCode err = U_ZERO_ERROR;
-							u_strToUTF8(&cbuffers[0][0], SI32(CG3_BUFFER_SIZE - 1), nullptr, ext->tag.c_str(), SI32(ext->tag.size()), &err);
+							u_strToUTF8(&cbuffers[0][0], SI32(CG3_BUFFER_SIZE - 1), nullptr, ext->tag.data(), SI32(ext->tag.size()), &err);
 
 							Process& es = externals[rule.varname];
 							try {
@@ -1269,7 +1278,7 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 
 							UChar dep_self[12] = {};
 							UChar dep_parent[12] = {};
-							if (u_sscanf(tter->tag.c_str(), "%[0-9cd]->%[0-9pm]", &dep_self, &dep_parent) == 2) {
+							if (u_sscanf(tter->tag.data(), "%[0-9cd]->%[0-9pm]", &dep_self, &dep_parent) == 2) {
 								if (dep_self[0] == 'c' || dep_self[0] == 'd') {
 									cohort_dep[i - 1].first = DEP_NO_PARENT;
 									if (rel_trg == DEP_NO_PARENT) {
@@ -1362,6 +1371,9 @@ uint32_t GrammarApplicator::runRulesOnSingleWindow(SingleWindow& current, const
 							current.cohorts.insert(current.cohorts.begin() + cohort->local_number + i + 1, cCohort);
 						}
 
+						// Move text from the to-be-deleted cohort to the last new cohort
+						std::swap(cohorts.back().first->text, cohort->text);
+
 						for (size_t i = 0; i < cohorts.size(); ++i) {
 							Cohort* cCohort = cohorts[i].first;
 
@@ -2442,7 +2454,7 @@ uint32_t GrammarApplicator::runGrammarOnSingleWindow(SingleWindow& current) {
 				for (size_t i = 1; i < current.cohorts.size(); ++i) {
 					Tag* t = current.cohorts[i]->wordform;
 					tag.assign(t->tag.begin() + 2, t->tag.begin() + t->tag.size() - 2);
-					u_fprintf(ux_stderr, " %S", tag.c_str());
+					u_fprintf(ux_stderr, " %S", tag.data());
 				}
 				u_fprintf(ux_stderr, "\n");
 				u_fflush(ux_stderr);
@@ -2566,7 +2578,7 @@ label_runGrammarOnWindow_begin:
 		for (size_t i = 1; i < current->cohorts.size(); ++i) {
 			Tag* t = current->cohorts[i]->wordform;
 			tag.assign(t->tag.begin() + 2, t->tag.begin() + t->tag.size() - 2);
-			u_fprintf(ux_stderr, " %S", tag.c_str());
+			u_fprintf(ux_stderr, " %S", tag.data());
 		}
 		u_fprintf(ux_stderr, "\n");
 		u_fflush(ux_stderr);
