@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -33,16 +33,16 @@ public:
 	void runGrammarOnText(std::istream& input, std::ostream& output);
 
 	bool getNullFlush();
-	bool wordform_case;
-	bool print_word_forms;
-	bool print_only_first;
+	bool wordform_case = false;
+	bool print_word_forms = true;
+	bool print_only_first = false;
 	void setNullFlush(bool pNullFlush);
 
 	void testPR(std::ostream& output);
 
 protected:
 	struct Node {
-		int self;
+		int self = 0;
 		UString lemma;
 		UString form;
 		UString pos;
@@ -53,8 +53,8 @@ protected:
 	std::map<int, Node> nodes;
 	std::map<int, std::vector<int>> deps;
 
-	bool nullFlush;
-	bool runningWithNullFlush;
+	bool nullFlush = false;
+	bool runningWithNullFlush = false;
 
 	void printReading(Reading* reading, Node& n, std::ostream& output);
 	void printSingleWindow(SingleWindow* window, std::ostream& output);
