@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -29,8 +29,8 @@ namespace CG3 {
 
 class NicelineApplicator : public virtual GrammarApplicator {
 private:
-	bool did_warn_statictags;
-	bool did_warn_subreadings;
+	bool did_warn_statictags = false;
+	bool did_warn_subreadings = false;
 
 public:
 	NicelineApplicator(std::ostream& ux_err);
