@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -88,36 +88,8 @@ void PlaintextApplicator::runGrammarOnText(std::istream& input, std::ostream& ou
 
 	while (!input.eof()) {
 		++lines;
-		size_t offset = 0, packoff = 0;
-		// Read as much of the next line as will fit in the current buffer
-		while (u_fgets(&line[offset], static_cast<int32_t>(line.size() - offset - 1), input)) {
-			// Copy the segment just read to cleaned
-			for (size_t i = offset; i < line.size(); ++i) {
-				// Only copy one space character, regardless of how many are in input
-				if (ISSPACE(line[i]) && !ISNL(line[i])) {
-					cleaned[packoff++] = ' ';
-					while (ISSPACE(line[i]) && !ISNL(line[i])) {
-						++i;
-					}
-				}
-				// Break if there is a newline
-				if (ISNL(line[i])) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					goto gotaline; // Oh how I wish C++ had break 2;
-				}
-				if (line[i] == 0) {
-					cleaned[packoff + 1] = cleaned[packoff] = 0;
-					break;
-				}
-				cleaned[packoff++] = line[i];
-			}
-			// If we reached this, buffer wasn't big enough. Double the size of the buffer and try again.
-			offset = line.size() - 2;
-			line.resize(line.size() * 2, 0);
-			cleaned.resize(line.size() + 1, 0);
-		}
+		auto packoff = get_line_clean(line, cleaned, input);
 
-	gotaline:
 		// Trim trailing whitespace
 		while (cleaned[0] && ISSPACE(cleaned[packoff - 1])) {
 			cleaned[packoff - 1] = 0;
@@ -164,7 +136,7 @@ void PlaintextApplicator::runGrammarOnText(std::istream& input, std::ostream& ou
 			}
 			if (cCohort && (cSWindow->cohorts.size() >= hard_limit || (!dep_delimit && grammar->delimiters && doesSetMatchCohortNormal(*cCohort, grammar->delimiters->number)))) {
 				if (!is_conv && cSWindow->cohorts.size() >= hard_limit) {
-					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.c_str(), numCohorts, numLines);
+					u_fprintf(ux_stderr, "Warning: Hard limit of %u cohorts reached at cohort %S (#%u) on line %u - forcing break.\n", hard_limit, cCohort->wordform->tag.data(), numCohorts, numLines);
 					u_fflush(ux_stderr);
 				}
 				for (auto iter : cCohort->readings) {
@@ -231,7 +203,7 @@ void PlaintextApplicator::runGrammarOnText(std::istream& input, std::ostream& ou
 					--len;
 				}
 				if (*p) {
-					tokens.insert(tokens.begin() + tkz, p);
+					tokens.insert(tokens.begin() + tkz, UnicodeString(p));
 				}
 			}
 
@@ -348,11 +320,11 @@ void PlaintextApplicator::printCohort(Cohort* cohort, std::ostream& output) {
 		return;
 	}
 
-	u_fprintf(output, "%.*S ", cohort->wordform->tag.size() - 4, cohort->wordform->tag.c_str() + 2);
+	u_fprintf(output, "%.*S ", cohort->wordform->tag.size() - 4, cohort->wordform->tag.data() + 2);
 }
 
 void PlaintextApplicator::printSingleWindow(SingleWindow* window, std::ostream& output) {
-	uint32_t cs = static_cast<uint32_t>(window->cohorts.size());
+	uint32_t cs = UI32(window->cohorts.size());
 	for (uint32_t c = 0; c < cs; c++) {
 		Cohort* cohort = window->cohorts[c];
 		printCohort(cohort, output);
