@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -38,9 +38,9 @@ public:
 	void relabel();
 
 private:
-	std::ostream* ux_stderr;
-	Grammar* grammar;
-	const Grammar* relabels;
+	std::ostream* ux_stderr = nullptr;
+	Grammar* grammar = nullptr;
+	const Grammar* relabels = nullptr;
 
 	typedef std::unordered_map<UString, UString, hash_ustring> UStringMap;
 	typedef std::unordered_map<UString, Set*, hash_ustring> UStringSetMap;
