@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -47,12 +47,12 @@ class Set {
 public:
 	CG3_IMPORTS static std::ostream* dump_hashes_out;
 
-	uint8_t type;
-	uint32_t line;
-	uint32_t hash;
-	uint32_t number;
-	mutable uint32_t num_fail, num_match;
-	mutable double total_time;
+	uint8_t type = 0;
+	uint32_t line = 0;
+	uint32_t hash = 0;
+	uint32_t number = 0;
+	mutable uint32_t num_fail = 0, num_match = 0;
+	mutable double total_time = 0;
 	UString name;
 
 	trie_t trie;
@@ -62,7 +62,7 @@ public:
 	uint32Vector set_ops;
 	uint32Vector sets;
 
-	Set();
+	Set() = default;
 	~Set() {
 		trie_delete(trie);
 		trie_delete(trie_special);
@@ -71,6 +71,9 @@ public:
 	void setName(uint32_t to = 0);
 	void setName(const UChar* to);
 	void setName(const UString& to);
+	void setName(const UStringView& to) {
+		return setName(to.data());
+	}
 
 	bool empty() const;
 	uint32_t rehash();
