@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -60,7 +60,7 @@ SingleWindow::~SingleWindow() {
 	#endif
 
 	if (cohorts.size() > 1) {
-		for (uint32FlatHashMap::iterator iter = parent->relation_map.begin(); iter != parent->relation_map.end();) {
+		for (auto iter = parent->relation_map.begin(); iter != parent->relation_map.end();) {
 			if (iter->second <= cohorts.back()->global_number) {
 				iter = parent->relation_map.erase(iter);
 			}
@@ -89,7 +89,7 @@ SingleWindow::~SingleWindow() {
 
 void SingleWindow::clear() {
 	if (cohorts.size() > 1) {
-		for (uint32FlatHashMap::iterator iter = parent->relation_map.begin(); iter != parent->relation_map.end();) {
+		for (auto iter = parent->relation_map.begin(); iter != parent->relation_map.end();) {
 			if (iter->second <= cohorts.back()->global_number) {
 				iter = parent->relation_map.erase(iter);
 			}
@@ -117,6 +117,7 @@ void SingleWindow::clear() {
 
 	number = 0;
 	has_enclosures = false;
+	flush_after = false;
 	next = nullptr;
 	previous = nullptr;
 	parent = nullptr;
@@ -135,7 +136,7 @@ void SingleWindow::clear() {
 }
 
 void SingleWindow::appendCohort(Cohort* cohort) {
-	cohort->local_number = (uint32_t)cohorts.size();
+	cohort->local_number = UI32(cohorts.size());
 	cohort->parent = this;
 
 	if (cohort->dep_self) {
