@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -67,7 +67,7 @@ enum : uint32_t {
 	T_ATTACHTO         = (1 << 19),
 	T_SPECIAL          = (1 << 20),
 	T_USED             = (1 << 21),
-	T_GRAMMAR          = (1 << 22),
+	//T_GRAMMAR          = (1 << 22), // No longer needed since switch to grammar-owned single_tags
 	T_SET              = (1 << 23),
 	T_VSTR             = (1 << 24),
 	T_ENCL             = (1 << 25),
@@ -82,21 +82,25 @@ class Tag {
 public:
 	CG3_IMPORTS static std::ostream* dump_hashes_out;
 
-	C_OPS comparison_op;
-	double comparison_val;
-	uint32_t type;
-	uint32_t comparison_hash;
-	uint32_t dep_self, dep_parent;
-	uint32_t hash;
-	uint32_t plain_hash;
-	uint32_t number;
-	uint32_t seed;
+	C_OPS comparison_op = OP_NOP;
+	double comparison_val = 0;
+	uint32_t type = 0;
+	uint32_t comparison_hash = 0;
+	uint32_t dep_self = 0;
+	union {
+		uint32_t dep_parent = 0;
+		uint32_t variable_hash;
+	};
+	uint32_t hash = 0;
+	uint32_t plain_hash = 0;
+	uint32_t number = 0;
+	uint32_t seed = 0;
 	UString tag;
 	std::unique_ptr<SetVector> vs_sets;
 	std::unique_ptr<UStringVector> vs_names;
-	mutable URegularExpression* regexp;
+	mutable URegularExpression* regexp = nullptr;
 
-	Tag();
+	Tag() = default;
 	Tag(const Tag& o);
 	~Tag();
 	void parseTagRaw(const UChar* to, Grammar* grammar);
