@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -79,6 +79,19 @@ SingleWindow* Window::allocAppendSingleWindow() {
 	return swindow;
 }
 
+SingleWindow* Window::back() {
+	if (!next.empty()) {
+		return next.back();
+	}
+	else if (current) {
+		return current;
+	}
+	else if (!previous.empty()) {
+		return previous.back();
+	}
+	return nullptr;
+}
+
 void Window::shuffleWindowsDown() {
 	if (current) {
 		current->variables_set = parent->variables;
