@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -126,21 +126,20 @@ public:
 		DEFAULT_CAP = static_cast<size_type>(16u),
 	};
 
-	flat_unordered_map()
-	  : size_(0)
-	{
-	}
-
 	size_t insert(const value_type& t) {
 		assert(t.first != res_empty && t.first != res_del && "Key cannot be res_empty or res_del!");
 
+		if (deleted && size_ + deleted == capacity()) {
+			reserve(capacity());
+		}
+
 		if ((size_ + 1) * 3 / 2 >= capacity() / 2) {
 			reserve(std::max(static_cast<size_type>(DEFAULT_CAP), capacity() * 2));
 		}
 		size_t max = capacity() - 1;
 		size_t spot = hash_value(t.first) & max;
 		while (elements[spot].first != res_empty && elements[spot].first != t.first) {
-			spot = (spot + 5) & max;
+			spot = hash_value_sz(spot) & max;
 		}
 		if (elements[spot].first != t.first) {
 			elements[spot] = t;
@@ -174,12 +173,18 @@ public:
 		size_t max = capacity() - 1;
 		size_t spot = hash_value(t) & max;
 		while (elements[spot].first != res_empty && elements[spot].first != t) {
-			spot = (spot + 5) & max;
+			spot = hash_value_sz(spot) & max;
 		}
 		if (elements[spot].first == t) {
 			elements[spot].first = res_del;
 			elements[spot].second = V();
 			--size_;
+			if (size_ == 0 && deleted != 0) {
+				clear();
+			}
+			else {
+				++deleted;
+			}
 		}
 	}
 
@@ -188,6 +193,12 @@ public:
 		elements[it.i].second = V();
 		++it;
 		--size_;
+		if (size_ == 0 && deleted != 0) {
+			clear();
+		}
+		else {
+			++deleted;
+		}
 		return it;
 	}
 
@@ -199,8 +210,8 @@ public:
 		if (size_) {
 			size_t max = capacity() - 1;
 			size_t spot = hash_value(t) & max;
-			while (elements[spot].first != res_empty && elements[spot].first != t) {
-				spot = (spot + 5) & max;
+			for (size_t i = 0; i < capacity() * 4 && elements[spot].first != res_empty && elements[spot].first != t; ++i) {
+				spot = hash_value_sz(spot) & max;
 			}
 			if (elements[spot].first == t) {
 				it.fus = this;
@@ -211,6 +222,14 @@ public:
 		return it;
 	}
 
+	const_iterator find(T t) {
+		if (deleted && size_ + deleted == capacity()) {
+			reserve(capacity());
+		}
+
+		return const_cast<const flat_unordered_map*>(this)->find(t);
+	}
+
 	size_t count(T t) const {
 		return (find(t) != end());
 	}
@@ -218,12 +237,16 @@ public:
 	V& operator[](const T& t) {
 		assert(t != res_empty && t != res_del && "Key cannot be res_empty or res_del!");
 
+		if (deleted && size_ + deleted == capacity()) {
+			reserve(capacity());
+		}
+
 		size_t at = std::numeric_limits<size_t>::max();
 		if (size_) {
 			size_t max = capacity() - 1;
 			size_t spot = hash_value(t) & max;
 			while (elements[spot].first != res_empty && elements[spot].first != t) {
-				spot = (spot + 5) & max;
+				spot = hash_value_sz(spot) & max;
 			}
 			if (elements[spot].first == t) {
 				at = spot;
@@ -263,27 +286,28 @@ public:
 	void reserve(size_type n) {
 		if (size_ == 0) {
 			elements.resize(n, std::make_pair(res_empty, V()));
+			deleted = 0;
 			return;
 		}
 
 		static container vals;
 		vals.resize(0);
 		vals.reserve(size_);
-		for (size_type i = 0, ie = capacity(); i < ie; ++i) {
-			if (elements[i].first != res_empty && elements[i].first != res_del) {
-				vals.push_back(elements[i]);
+		for (auto& elem : elements) {
+			if (elem.first != res_empty && elem.first != res_del) {
+				vals.push_back(elem);
 			}
 		}
 
 		clear(n);
 		size_ = vals.size();
 		size_t max = capacity() - 1;
-		for (size_type i = 0, ie = vals.size(); i < ie; ++i) {
-			size_t spot = hash_value(vals[i].first) & max;
-			while (elements[spot].first != res_empty && elements[spot].first != vals[i].first) {
-				spot = (spot + 5) & max;
+		for (auto& val : vals) {
+			size_t spot = hash_value(val.first) & max;
+			while (elements[spot].first != res_empty && elements[spot].first != val.first) {
+				spot = hash_value_sz(spot) & max;
 			}
-			elements[spot] = vals[i];
+			elements[spot] = val;
 		}
 	}
 
@@ -299,6 +323,7 @@ public:
 
 	void swap(flat_unordered_map& other) {
 		std::swap(size_, other.size_);
+		std::swap(deleted, other.deleted);
 		elements.swap(other.elements);
 	}
 
@@ -307,6 +332,7 @@ public:
 		elements.resize(0);
 		elements.resize(std::max(size_, n), std::make_pair(res_empty, V()));
 		size_ = 0;
+		deleted = 0;
 	}
 
 	container& get() {
@@ -314,11 +340,16 @@ public:
 	}
 
 private:
-	size_type size_;
+	size_type size_ = 0;
+	size_type deleted = 0;
 	container elements;
 
-	T hash_value(T t) const {
-		return (t << 8) | ((t >> 8) & 0xFF);
+	size_type hash_value_sz(size_type t) const {
+		return t * 3663850746527583589ull + 11210403176660999867ull;
+	}
+
+	size_type hash_value(T t) const {
+		return hash_value_sz(static_cast<size_type>(t));
 	}
 
 	friend class const_iterator;
