@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -127,21 +127,20 @@ public:
 		DEFAULT_CAP = static_cast<size_type>(16u),
 	};
 
-	flat_unordered_set()
-	  : size_(0)
-	{
-	}
-
 	void insert(T t) {
 		assert(t != res_empty && t != res_del && "Value cannot be res_empty or res_del!");
 
+		if (deleted && size_ + deleted == capacity()) {
+			reserve(capacity());
+		}
+
 		if ((size_ + 1) * 3 / 2 >= capacity() / 2) {
 			reserve(std::max(static_cast<size_type>(DEFAULT_CAP), capacity() * 2));
 		}
 		size_t max = capacity() - 1;
 		size_t spot = hash_value(t) & max;
 		while (elements[spot] != res_empty && elements[spot] != t) {
-			spot = (spot + 5) & max;
+			spot = hash_value_sz(spot) & max;
 		}
 		if (elements[spot] != t) {
 			elements[spot] = t;
@@ -174,11 +173,17 @@ public:
 		size_t max = capacity() - 1;
 		size_t spot = hash_value(t) & max;
 		while (elements[spot] != res_empty && elements[spot] != t) {
-			spot = (spot + 5) & max;
+			spot = hash_value_sz(spot) & max;
 		}
 		if (elements[spot] == t) {
 			elements[spot] = res_del;
 			--size_;
+			if (size_ == 0 && deleted != 0) {
+				clear();
+			}
+			else {
+				++deleted;
+			}
 		}
 	}
 
@@ -186,6 +191,12 @@ public:
 		elements[it.i] = res_del;
 		++it;
 		--size_;
+		if (size_ == 0 && deleted != 0) {
+			clear();
+		}
+		else {
+			++deleted;
+		}
 		return it;
 	}
 
@@ -197,8 +208,8 @@ public:
 		if (size_) {
 			size_t max = capacity() - 1;
 			size_t spot = hash_value(t) & max;
-			while (elements[spot] != res_empty && elements[spot] != t) {
-				spot = (spot + 5) & max;
+			for (size_t i = 0; i < capacity() * 4 && elements[spot] != res_empty && elements[spot] != t; ++i) {
+				spot = hash_value_sz(spot) & max;
 			}
 			if (elements[spot] == t) {
 				it.fus = this;
@@ -209,6 +220,14 @@ public:
 		return it;
 	}
 
+	const_iterator find(T t) {
+		if (deleted && size_ + deleted == capacity()) {
+			reserve(capacity());
+		}
+
+		return const_cast<const flat_unordered_set*>(this)->find(t);
+	}
+
 	size_t count(T t) const {
 		return (find(t) != end());
 	}
@@ -240,27 +259,28 @@ public:
 	void reserve(size_type n) {
 		if (size_ == 0) {
 			elements.resize(n, res_empty);
+			deleted = 0;
 			return;
 		}
 
 		static thread_local container vals;
 		vals.resize(0);
 		vals.reserve(size_);
-		for (size_type i = 0, ie = capacity(); i < ie; ++i) {
-			if (elements[i] != res_empty && elements[i] != res_del) {
-				vals.push_back(elements[i]);
+		for (auto& elem : elements) {
+			if (elem != res_empty && elem != res_del) {
+				vals.push_back(elem);
 			}
 		}
 
 		clear(n);
 		size_ = vals.size();
 		size_t max = capacity() - 1;
-		for (size_type i = 0, ie = vals.size(); i < ie; ++i) {
-			size_t spot = hash_value(vals[i]) & max;
-			while (elements[spot] != res_empty && elements[spot] != vals[i]) {
-				spot = (spot + 5) & max;
+		for (auto& val : vals) {
+			size_t spot = hash_value(val) & max;
+			while (elements[spot] != res_empty && elements[spot] != val) {
+				spot = hash_value_sz(spot) & max;
 			}
-			elements[spot] = vals[i];
+			elements[spot] = val;
 		}
 	}
 
@@ -276,6 +296,7 @@ public:
 
 	void swap(flat_unordered_set& other) {
 		std::swap(size_, other.size_);
+		std::swap(deleted, other.deleted);
 		elements.swap(other.elements);
 	}
 
@@ -284,6 +305,7 @@ public:
 		elements.resize(0);
 		elements.resize(std::max(size_, n), res_empty);
 		size_ = 0;
+		deleted = 0;
 	}
 
 	container& get() {
@@ -291,11 +313,16 @@ public:
 	}
 
 private:
-	size_type size_;
+	size_type size_ = 0;
+	size_type deleted = 0;
 	container elements;
 
-	T hash_value(T t) const {
-		return (t << 8) | ((t >> 8) & 0xFF);
+	size_type hash_value_sz(size_type t) const {
+		return t * 3663850746527583589ull + 11210403176660999867ull;
+	}
+
+	size_type hash_value(T t) const {
+		return hash_value_sz(static_cast<size_type>(t));
 	}
 
 	friend class const_iterator;
