@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -262,13 +262,16 @@ public:
 		if (it == elements.end()) {
 			return end();
 		}
-		if (it->ub < t || it->lb > t) {
+		if (it->ub < t) {
 			++it;
 			if (it == elements.end()) {
 				return end();
 			}
 			t = it->lb;
 		}
+		if (it->lb > t) {
+			t = it->lb;
+		}
 		return const_iterator(elements, it, t);
 	}
 
