@@ -41,6 +41,7 @@ enum OPTIONS {
 	OUT_CG,
 	OUT_CG2,
 	OUT_APERTIUM,
+	OUT_FST,
 	OUT_MATXIN,
 	OUT_NICELINE,
 	OUT_PLAIN,
@@ -72,6 +73,7 @@ std::array<UOption, NUM_OPTIONS> options{
 	UOPTION_DEF_D("out-cg",       'C', UOPT_NO_ARG,       "sets output format to CG (default)"),
 	UOPTION_DEF_D("V",            'V', UOPT_NO_ARG,       nullptr),
 	UOPTION_DEF_D("out-apertium", 'A', UOPT_NO_ARG,       "sets output format to Apertium"),
+	UOPTION_DEF_D("out-fst",      'F', UOPT_NO_ARG,       "sets output format to HFST/XFST"),
 	UOPTION_DEF_D("out-matxin",   'M', UOPT_NO_ARG,       "sets output format to Matxin"),
 	UOPTION_DEF_D("out-niceline", 'N', UOPT_NO_ARG,       "sets output format to Niceline CG"),
 	UOPTION_DEF_D("out-plain",    'X', UOPT_NO_ARG,       "sets output format to plain text"),
