@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2007-2020, GrammarSoft ApS
+* Copyright (C) 2007-2021, GrammarSoft ApS
 * Developed by Tino Didriksen <mail@tinodidriksen.com>
 * Design by Eckhard Bick <eckhard.bick@mail.dk>, Tino Didriksen <mail@tinodidriksen.com>
 *
@@ -188,6 +188,14 @@ public:
 		return elements.end();
 	}
 
+	const_iterator cbegin() const {
+		return elements.cbegin();
+	}
+
+	const_iterator cend() const {
+		return elements.cend();
+	}
+
 	const_reverse_iterator rbegin() const {
 		return elements.rbegin();
 	}
