@@ -175,12 +175,16 @@ inline bool ux_simplecasecmp(const UChar* a, const UChar* b, const size_t n) {
 }
 
 inline bool ux_simplecasecmp(const UChar* a, const UString& b) {
-	return ux_simplecasecmp(a, b.c_str(), b.size());
+	return ux_simplecasecmp(a, b.data(), b.size());
+}
+
+inline bool ux_simplecasecmp(const UChar* a, const UStringView& b) {
+	return ux_simplecasecmp(a, b.data(), b.size());
 }
 
 inline bool ux_strCaseCompare(const UString& a, const UString& b) {
 	UErrorCode status = U_ZERO_ERROR;
-	auto rv = u_strCaseCompare(a.c_str(), SI32(a.size()), b.c_str(), SI32(b.size()), U_FOLD_CASE_DEFAULT, &status);
+	auto rv = u_strCaseCompare(a.data(), SI32(a.size()), b.data(), SI32(b.size()), U_FOLD_CASE_DEFAULT, &status);
 	if (status != U_ZERO_ERROR) {
 		throw new std::runtime_error(u_errorName(status));
 	}
@@ -207,13 +211,13 @@ struct substr_t {
 
 	~substr_t() {
 		if (count != Str::npos) {
-			value_type* buf = const_cast<value_type*>(str.c_str() + offset);
+			value_type* buf = const_cast<value_type*>(str.data() + offset);
 			buf[count] = old_value;
 		}
 	}
 
-	const value_type* c_str() const {
-		value_type* buf = const_cast<value_type*>(str.c_str() + offset);
+	const value_type* data() const {
+		value_type* buf = const_cast<value_type*>(str.data() + offset);
 		buf[count] = 0;
 		return buf;
 	}
