@@ -13,11 +13,16 @@ if (!$binary_mwesplit || $binary_mwesplit eq '' || !(-x $binary_mwesplit)) {
 	die("Error: $binary_mwesplit is not executable!");
 }
 
+my $bad = 0;
+
 `"$binary_mwesplit" < input.txt > output.txt 2>>stderr.txt`;
 `diff -ZB expected.txt output.txt >diff.txt`;
 
 if (-s "diff.txt") {
 	print STDERR "Fail.\n";
+	$bad = 1;
 } else {
 	print STDERR "Success Success.\n";
 }
+
+exit($bad);
